﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema

Public Class category_v01
    Inherits model_base

    'Interne Variablen
    Dim v_cat_id As Integer
    Dim v_bezeichnung As String
    Dim v_color As String = "#000000"
    Dim v_symbol As Byte()
    Dim v_aktiv As Nullable(Of Boolean) = True
    Dim v_parent_cat_id As Nullable(Of Integer)

#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                Case "bezeichnung"
                    If String.IsNullOrEmpty(bezeichnung) Then result = "Bezeichnung darf nicht leer sein"
            End Select

            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If
            NotifyPropertyChanged("hasErrors")
            Return result
        End Get
    End Property
#End Region

#Region "Mapped"
    <Key>
    Public Property cat_id As Integer
        Get
            Return v_cat_id
        End Get
        Set(value As Integer)
            If v_cat_id <> value Then
                v_cat_id = value
                NotifyPropertyChanged("cat_id")
            End If
        End Set
    End Property

    Public Property bezeichnung As String
        Get
            Return v_bezeichnung
        End Get
        Set(value As String)
            If v_bezeichnung <> value Then
                v_bezeichnung = value
                NotifyPropertyChanged("bezeichnung")
            End If
        End Set
    End Property

    Public Property color As String
        Get
            Return v_color
        End Get
        Set(value As String)
            If v_color <> value Then
                v_color = value
                NotifyPropertyChanged("color")
                NotifyPropertyChanged("solid_color")
            End If
        End Set
    End Property

    <MaxLength>
    Public Property symbol As Byte()
        Get
            Return v_symbol
        End Get
        Set(value As Byte())
            If v_symbol IsNot value Then
                v_symbol = value
                NotifyPropertyChanged("symbol")
                NotifyPropertyChanged("symbol_ever")
            End If
        End Set
    End Property

    Public Property aktiv As Nullable(Of Boolean)
        Get
            Return v_aktiv
        End Get
        Set(value As Nullable(Of Boolean))
            If Not v_aktiv.Equals(value) Then
                v_aktiv = value
                NotifyPropertyChanged("aktiv")
            End If
        End Set
    End Property

    Public Property parent_cat_id As Nullable(Of Integer)
        Get
            'Debug.Print("parent_cat_id: " & v_parent_cat_id.ToString)
            Return v_parent_cat_id
        End Get
        Set(value As Nullable(Of Integer))
            'If v_parent_cat_id <> value Then
            v_parent_cat_id = value
                NotifyPropertyChanged("parent_cat_id")
            'End If
        End Set
    End Property
#End Region

#Region "Navigation Properties"
    Dim v_nv_subcats_back As category_v01
    Public Overridable Property nv_subcats As New ObservableCollection(Of category_v01)
    <ForeignKey("parent_cat_id")>
    Public Property nv_subcats_back As category_v01
        Get
            Return v_nv_subcats_back
        End Get
        Set(value As category_v01)
            If v_nv_subcats_back IsNot value Then
                v_nv_subcats_back = value
                NotifyPropertyChanged("nv_subcats_back")
            End If
        End Set
    End Property
    Public Overridable Property nv_cat_trans2kat As New ObservableCollection(Of trans_2_kat)
    Public Overridable Property transrep_2_kat2 As New ObservableCollection(Of transrep_2_kat)
    Public Overridable Property nv_category_2_budcat As New ObservableCollection(Of budget_2_cat_v01)
    Public Overridable Property nv_cat_2_transtemp As New ObservableCollection(Of transtemp_2_kat)
#End Region

#Region "Extension"
    Public ReadOnly Property solid_color() As SolidColorBrush
        Get
            If color Is Nothing Then
                Return ColorBrushFromResources("IdealForegroundColorBrush")
            Else
                Try
                    Dim v_col As System.Drawing.Color = System.Drawing.ColorTranslator.FromHtml(color)
                    Dim v_color As Color = System.Windows.Media.Color.FromArgb(v_col.A, v_col.R, v_col.G, v_col.B)
                    Return New SolidColorBrush(v_color)
                Catch ex As Exception
                    Return ColorBrushFromResources("IdealForegroundColorBrush")
                End Try
            End If
        End Get
    End Property

    Public ReadOnly Property symbol_ever() As Byte()
        Get
            If symbol Is Nothing Then
                Return ResourceDictionaryImagetoByte("Category")
            Else
                Return symbol
            End If
        End Get
    End Property

    ''' <summary>
    ''' returns the entire name of a category including parent category name
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property fullname As String
        Get
            If parent_cat_id Is Nothing Then
                Return bezeichnung
            Else
                Return String.Concat(nv_subcats_back.bezeichnung, " - ", bezeichnung)
            End If
        End Get
    End Property

    ''' <summary>
    ''' Calculates the total amount of a category
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property totamount As Decimal
        Get
            If nv_cat_trans2kat Is Nothing OrElse nv_cat_trans2kat.Count = 0 Then
                Return 0
            Else
                Dim v_pos_value As Decimal = nv_cat_trans2kat.Where(Function(x) x.nv_trans_t2k_back.nv_transtyp_trans_back.internal_name = "ein").Sum(Function(x) x.amount)
                Dim v_neg_value As Decimal = nv_cat_trans2kat.Where(Function(x) x.nv_trans_t2k_back.nv_transtyp_trans_back.internal_name = "aus").Sum(Function(x) x.amount)
                Return v_pos_value - v_neg_value
            End If
        End Get
    End Property

    ''' <summary>
    ''' Returns the totamount of a category depending on a specified timespan
    ''' also sets the totamount_timespan property to the result.
    ''' e.g. if you calcutlate a graph which is not bound directly to the model to can use the totamount_timespan property as a temporary storage for the totamount to enable sorting.
    ''' </summary>
    ''' <param name="v_begin">Begindate must be smaller or equal to end date</param>
    ''' <param name="v_end">Enddate muust be greater or equal to begin date</param>
    ''' <returns>Decimal Value of Totalamount of a category</returns>
    Public Function totamount_timespan_caluclator(v_begin As Date, v_end As Date) As Decimal
        v_totamount_timespan = 0
        If v_begin = Nothing Then Return 0
        If v_end = Nothing Then Return 0

        If nv_cat_trans2kat Is Nothing OrElse nv_cat_trans2kat.Count = 0 Then
            Return 0
        Else
            Dim v_pos_value As Decimal = nv_cat_trans2kat.Where(Function(x) x.nv_trans_t2k_back.nv_transtyp_trans_back.internal_name = "ein" AndAlso x.nv_trans_t2k_back.datum >= v_begin AndAlso x.nv_trans_t2k_back.datum <= v_end).Sum(Function(x) x.amount)
            Dim v_neg_value As Decimal = nv_cat_trans2kat.Where(Function(x) x.nv_trans_t2k_back.nv_transtyp_trans_back.internal_name = "aus" AndAlso x.nv_trans_t2k_back.datum >= v_begin AndAlso x.nv_trans_t2k_back.datum <= v_end).Sum(Function(x) x.amount)
            v_totamount_timespan = v_pos_value - v_neg_value
            NotifyPropertyChanged("totamount_timespan")
            Return v_pos_value - v_neg_value
        End If
    End Function

    Dim v_totamount_timespan As Decimal = 0
    Public ReadOnly Property totamount_timespan As Decimal
        Get
            Return v_totamount_timespan
        End Get
    End Property


#End Region
End Class
