﻿Imports System.ComponentModel.DataAnnotations
Imports System.Collections.ObjectModel
Imports System.ComponentModel.DataAnnotations.Schema
''' <summary>
''' Represents the "new" searchitem for a search in HHHB3
''' </summary>
Public Class searchitem_v01
    Inherits model_base

    Dim v_id As Integer
    Dim v_left_parenthesis As String
    Dim v_right_parenthesis As String
    Dim v_entity_str As String
    'search values:
    'search value for IDs:
    Dim v_search_value_int As Integer
    'search_value for strings:
    Dim v_search_value_str As String
    'search_value for decimals:
    Dim v_search_value_dec As Decimal = 0D
    'search_value for dates:
    Dim v_search_value_date As Date = Date.Now
    Dim v_comparer As String
    Dim v_link As String
    Dim v_aktiv As Nullable(Of Boolean) = True
    'ID of the Search:
    Dim v_search_id As Integer

    Public Sub NotifyAll()
        NotifyPropertyChanged("left_parenthesis")
        NotifyPropertyChanged("right_parenthesis")
        NotifyPropertyChanged("entity_str")
        NotifyPropertyChanged("search_value_int")
        NotifyPropertyChanged("search_value_str")
        NotifyPropertyChanged("search_value_dec")
        NotifyPropertyChanged("search_value_date")
        NotifyPropertyChanged("comparer")
        NotifyPropertyChanged("link")
        NotifyPropertyChanged("aktiv")
        NotifyPropertyChanged("search_value_type")
        NotifyPropertyChanged("search_value_display")
        NotifyPropertyChanged("search_term")
    End Sub

#Region "Mapped"
    <Key>
    Public Property Id As Integer
        Get
            Return v_Id
        End Get
        Set(value As Integer)
            If v_Id <> value Then
                v_Id = value
                NotifyPropertyChanged("Id")
            End If
        End Set
    End Property

    Public Property left_parenthesis As String
        Get
            Return v_left_parenthesis
        End Get
        Set(value As String)
            If v_left_parenthesis <> value Then
                v_left_parenthesis = value
                NotifyPropertyChanged("left_parenthesis")
            End If
        End Set
    End Property

    Public Property right_parenthesis As String
        Get
            Return v_right_parenthesis
        End Get
        Set(value As String)
            If v_right_parenthesis <> value Then
                v_right_parenthesis = value
                NotifyPropertyChanged("right_parenthesis")
            End If
        End Set
    End Property

    Public Property entity_str As String
        Get
            Return v_entity_str
        End Get
        Set(value As String)
            If v_entity_str <> value Then
                v_entity_str = value
                'if entity changed then reset the search values:
                search_value_date = Date.Now
                search_value_dec = 0D
                search_value_int = Nothing
                search_value_str = String.Empty
                comparer = String.Empty

                NotifyPropertyChanged("comparer")
                NotifyPropertyChanged("entity_str")
                NotifyPropertyChanged("search_value_type")
                NotifyPropertyChanged("search_value_display")

            End If
        End Set
    End Property


    '"trans_status_v01" => int
    '"trans_typ_v01" => int
    '"konto_v01" => int
    '"to_konto_v01" => int
    '"receiver_v01" => int
    '"person_v01" => int
    '"tags_v01" => int
    '"category_v01" =>int
    '"note" => str
    '"date" => date
    '"value" => dec

    'Defines the valid search_value depending on the selected entity_str
    Public ReadOnly Property search_value_type As String
        Get
            Select Case entity_str
                Case "note"
                    Return "search_value_str"
                Case "date"
                    Return "search_value_date"
                Case "value"
                    Return "search_value_dec"
                Case Else
                    Return "search_value_int"
            End Select
        End Get
    End Property

    Public ReadOnly Property search_value_display As String
        Get
            Select Case search_value_type
                Case "search_value_str"
                    Return search_value_str
                Case "search_value_date"
                    Return search_value_date.ToShortDateString
                Case "search_value_dec"
                    Return String.Format("{0:C2}", search_value_dec)
                Case Else
                    'Return the value after searching the according entry to an id
                    Dim v_viewmodel As HHHB3.ViewModel.MainViewModel = DirectCast(DirectCast(Application.Current.MainWindow, MahApps.Metro.Controls.MetroWindow).DataContext, HHHB3.ViewModel.MainViewModel)
                    If v_viewmodel Is Nothing Then Return String.Empty
                    Return v_viewmodel.SearchItemView(entity_str, search_value_int)
            End Select
            Return String.Empty
        End Get
    End Property

    ''' <summary>
    ''' Returns the search_term of the searchitem for using in SQL Query
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property search_term As String
        Get
            Dim v_searchstring As String = left_parenthesis

            Select Case search_value_type
                Case "search_value_str"
                    'The only possible use is to search note:
                    v_searchstring &= "comment " & comparer & " '" & search_value_str & "'"
                Case "search_value_date"
                    v_searchstring &= "datum " & comparer & " '" & search_value_date.Month.ToString & "/" & search_value_date.Day.ToString & "/" & search_value_date.Year.ToString & "'"
                Case "search_value_dec"
                    'Decimal is not available at the moment
                    v_searchstring &= " (sum(trans_2_kat.amount) " & comparer & " " & search_value_dec & ")"
                Case Else
                    'Creating searchstring depending on selected entity:
                    Select Case entity_str
                        Case "trans_status_v01"
                            v_searchstring &= "transstatus_id "
                        Case "trans_typ_v01"
                            v_searchstring &= "transtyp_id "
                        Case "konto_v01"
                            v_searchstring &= "konto_id "
                        Case "to_konto_v01"
                            v_searchstring &= "to_konto_id "
                        Case "receiver_v01"
                            v_searchstring &= "receiver_id "
                        Case "person_v01"
                            v_searchstring &= "person_id "
                        Case "tags_v01"
                            v_searchstring &= "trans_tag_v01.tagid "
                        Case "category_v01"
                            v_searchstring &= "trans_2_kat.category_id "
                    End Select
                    v_searchstring &= comparer & " " & search_value_int.ToString
            End Select
            'End of searchstring:
            v_searchstring &= " " & right_parenthesis & " " & link & " "
            Return v_searchstring
        End Get
    End Property

    Public Property search_value_int As Integer
        Get
            Return v_search_value_int
        End Get
        Set(value As Integer)
            If v_search_value_int <> value Then
                v_search_value_int = value
                NotifyPropertyChanged("search_value_int")
                NotifyPropertyChanged("search_value_display")
            End If
        End Set
    End Property

    Public Property search_value_str As String
        Get
            Return v_search_value_str
        End Get
        Set(value As String)
            If v_search_value_str <> value Then
                v_search_value_str = value
                NotifyPropertyChanged("search_value_str")
                NotifyPropertyChanged("search_value_display")
            End If
        End Set
    End Property

    Public Property search_value_dec As Decimal
        Get
            Return v_search_value_dec
        End Get
        Set(value As Decimal)
            If v_search_value_dec <> value Then
                v_search_value_dec = value
                NotifyPropertyChanged("search_value_dec")
                NotifyPropertyChanged("search_value_display")
            End If
        End Set
    End Property

    Public Property search_value_date As Date
        Get
            Return v_search_value_date
        End Get
        Set(value As Date)
            If v_search_value_date <> value Then
                v_search_value_date = value
                NotifyPropertyChanged("search_value_date")
                NotifyPropertyChanged("search_value_display")
            End If
        End Set
    End Property



    Public Property comparer As String
        Get
            Return v_comparer
        End Get
        Set(value As String)
            If v_comparer <> value Then
                v_comparer = value
                NotifyPropertyChanged("comparer")
            End If
        End Set
    End Property
    Public Property link As String
        Get
            Return v_link
        End Get
        Set(value As String)
            If v_link <> value Then
                v_link = value
                NotifyPropertyChanged("link")
            End If
        End Set
    End Property

    Public Property aktiv As Nullable(Of Boolean)
        Get
            Return v_aktiv
        End Get
        Set(value As Nullable(Of Boolean))
            If Not v_aktiv.Equals(value) Then
                v_aktiv = value
                NotifyPropertyChanged("aktiv")
            End If
        End Set
    End Property


    Public Property search_id As Integer
        Get
            Return v_search_id
        End Get
        Set(value As Integer)
            If v_search_id <> value Then
                v_search_id = value
                NotifyPropertyChanged("search_id")
            End If
        End Set
    End Property
#End Region

#Region "Navigation Properties"
    Dim v_nv_searchdefinition As searchdefinition_v01

    <ForeignKey("search_id")>
    Public Property nv_searchdefinition As searchdefinition_v01
        Get
            Return v_nv_searchdefinition
        End Get
        Set(value As searchdefinition_v01)
            If v_nv_searchdefinition IsNot value Then
                v_nv_searchdefinition = value
                NotifyPropertyChanged("nv_searchdefinition")
            End If
        End Set
    End Property

#End Region

#Region "ErrorInterface"
    ''' <summary>
    ''' Routine für die Fehlervalidierung
    ''' </summary>
    ''' <param name="columnName"></param>
    ''' <returns></returns>
    Public Overrides ReadOnly Property ErrorItem(columnName As String) As String
        Get
            Dim result As String = String.Empty
            Select Case columnName
                'Case "search_value"
                'If String.IsNullOrEmpty(v_search_value) Then result = "Wert darf nicht leer sein"
            End Select
            If String.IsNullOrEmpty(result) Then
                v_haserrors = False
            Else
                v_haserrors = True
            End If
            NotifyPropertyChanged("hasErrors")
            Return result
        End Get
    End Property
#End Region


End Class
