﻿Imports System.ComponentModel
Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.Reflection
Imports System.Runtime.CompilerServices
Imports System.ComponentModel.DataAnnotations
Imports System.ComponentModel.DataAnnotations.Schema
Imports System.Runtime.Serialization
''' <summary>
''' Base Klasse für Modelle die Propertychanged und die IEditable enthält
''' </summary>
Public Class model_base
    Implements INotifyPropertyChanged
    Implements IEditableObject
    Implements IDisposable
    Implements INotifyCollectionChanged
    Implements IDataErrorInfo

    Public _Collections As INotifyCollectionChanged()

    Public Event CollectionChanged As NotifyCollectionChangedEventHandler Implements INotifyCollectionChanged.CollectionChanged
    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

    Public Sub NotifyPropertyChanged(<CallerMemberName()> Optional ByVal propertyName As String = Nothing)
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
    End Sub

#Region "Edit Routinen"
    Private backup_props As Hashtable
    Private v_isinEdit As Boolean = False

    <NotMapped>
    Public Property isinEdit As Boolean
        Get
            Return v_isinEdit
        End Get
        Private Set(value As Boolean)
            If v_isinEdit <> value Then
                v_isinEdit = value
                NotifyPropertyChanged("isinEdit")
            End If
        End Set
    End Property


    Public Overridable Sub BeginEdit() Implements IEditableObject.BeginEdit
        'enumerate properties
        Dim properties As PropertyInfo() = (Me.[GetType]()).GetProperties(BindingFlags.[Public] Or BindingFlags.Instance)

        backup_props = New Hashtable(properties.Length - 1)

        For i As Integer = 0 To properties.Length - 1
            'check if there is set accessor
            If properties(i).GetSetMethod() IsNot Nothing Then
                Dim value As Object = properties(i).GetValue(Me, Nothing)
                backup_props.Add(properties(i).Name, value)
            End If
        Next
        isinEdit = True
    End Sub

    Public Overridable Sub EndEdit() Implements IEditableObject.EndEdit
        backup_props = Nothing
        isinEdit = False
    End Sub

    Public Overridable Sub CancelEdit() Implements IEditableObject.CancelEdit
        'check for inappropriate call sequence
        If backup_props Is Nothing Then
            Return
        End If

        'restore old values
        Dim properties As PropertyInfo() = (Me.[GetType]()).GetProperties(BindingFlags.[Public] Or BindingFlags.Instance)
        For i As Integer = 0 To properties.Length - 1
            'check if there is set accessor
            If properties(i).GetSetMethod() IsNot Nothing Then
                Dim value As Object = backup_props(properties(i).Name)
                properties(i).SetValue(Me, value, Nothing)
            End If
        Next

        'delete current values
        backup_props = Nothing
        isinEdit = False
    End Sub

#End Region

#Region "IDisposable Support"
    Private disposedValue As Boolean ' To detect redundant calls

    ' IDisposable
    Protected Overridable Sub Dispose(disposing As Boolean)
        If Not disposedValue Then
            If disposing Then
                ' TODO: dispose managed state (managed objects).
            End If

            ' TODO: free unmanaged resources (unmanaged objects) and override Finalize() below.
            ' TODO: set large fields to null.
        End If
        disposedValue = True
    End Sub

    ' TODO: override Finalize() only if Dispose(disposing As Boolean) above has code to free unmanaged resources.
    'Protected Overrides Sub Finalize()
    '    ' Do not change this code.  Put cleanup code in Dispose(disposing As Boolean) above.
    '    Dispose(False)
    '    MyBase.Finalize()
    'End Sub

    ' This code added by Visual Basic to correctly implement the disposable pattern.
    Public Sub Dispose() Implements IDisposable.Dispose
        ' Do not change this code.  Put cleanup code in Dispose(disposing As Boolean) above.
        Dispose(True)
        ' TODO: uncomment the following line if Finalize() is overridden above.
        ' GC.SuppressFinalize(Me)
    End Sub
#End Region

    Private Sub Collection_CollectionChanged(sender As Object, e As NotifyCollectionChangedEventArgs)
        Debug.Print("model_base collectionchanged")
    End Sub


#Region "IDataErrorInfo"
    Public Overridable ReadOnly Property ErrorItem(columnName As String) As String Implements IDataErrorInfo.Item
        Get
            Throw New NotImplementedException()
        End Get
    End Property

    Public ReadOnly Property [Error] As String Implements IDataErrorInfo.Error
        Get
            Return ""
            'Throw New NotImplementedException()
        End Get
    End Property


    Public v_haserrors As Boolean = True
    Public ReadOnly Property hasErrors As Boolean
        Get

            Return v_haserrors
        End Get
    End Property

#End Region

End Class
