﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized

Namespace HHHB3.ViewModel
    Partial Public Class MainViewModel
#Region "DocWatcher"

        Dim doc_watcher As New IO.FileSystemWatcher
        ''' <summary>
        ''' Initialisiert den DocWatcher wenn dieser aktiviert werden soll:
        ''' Initiates the Document Watcher if it is activated in settings
        ''' </summary>
        Private Sub InitDocWatcher()
            Try
                'Check if the path is valid:
                If String.IsNullOrEmpty(SettingsCurrent.doc_watcher_path) OrElse IO.Directory.Exists(SettingsCurrent.doc_watcher_path) = False Then
                    MessageAdd(New message_v01("Dokumentenüberwachung", "wird nicht ausgeführt. Grund: Pfad leer oder ungültig", False))
                Else
                    With doc_watcher
                        .Path = SettingsCurrent.doc_watcher_path
                        .Filter = "*.*"
                        .IncludeSubdirectories = True
                        '.NotifyFilter = IO.NotifyFilters.LastWrite 'nur wenn eine neue Datei angelegt wird reagieren
                        AddHandler .Created, AddressOf docwatcher_newfile
                        .EnableRaisingEvents = True
                        cls_logger.v_logger.Info("InitDocwatcher: EnableRaisingEvents is set to true with path: " & SettingsCurrent.doc_watcher_path)
                        Dim v_header As String = "Dokumentenüberwachung"
                        Dim v_msg As String = "wird ausgeführt"
                        ctoast.define_toaster(v_header, v_msg, Windows.UI.Notifications.ToastTemplateType.ToastImageAndText04, True)
                        MessageAdd(New message_v01(v_header, v_msg, False))
                        v_header = Nothing
                        v_msg = Nothing
                    End With
                End If
            Catch ex As Exception
                cls_logger.v_logger.Error("InitDocWatcher failed with:" & ex.Message.ToString)
                Debug.Print(ex.Message)
            End Try

        End Sub

        Private Sub docwatcher_newfile(ByVal source As Object, ByVal e As IO.FileSystemEventArgs)
            'Exectued when a new file is created:
            'Soll nur benachrichtigt werden oder soll auch eine Buchung angelegt werden:
            'Check if only a message should be displayed or a transaction should be created:
            If SettingsCurrent.doc_watcher_notify Then
                'Nur benachrichtigen
                'notifyicon.show
                'Messages.Add(New Model_Message("Neues Dokument", Model_Message.Actions.NewDoc, Model_Message.Styles.Information, "hier klicken um Buchung zu erstellen"))
                Me.dispatcher.Invoke(Sub()
                                         ctoast.define_toaster("Dokumentenüberwachung", "Neues Dokument wurde gefunden!", Windows.UI.Notifications.ToastTemplateType.ToastImageAndText04, True)
                                     End Sub)
            Else
                'Buchung erstellen:
                Me.dispatcher.Invoke(Sub()
                                         ctoast.define_toaster("Dokumentenüberwachung", "Neues Dokument wurde gefunden!", Windows.UI.Notifications.ToastTemplateType.ToastImageAndText04, True, "Neue Buchung wird angelegt")
                                         TransactionAdd(, New documents_v01 With {.bezeichnung = "Empfangenes Dokument", .path = e.FullPath})
                                     End Sub)

            End If
        End Sub

        Dim v_docwatcherstate As Boolean = False
        ''' <summary>
        ''' Indicates the EnableRaisingEvent Status of the DocWatcher
        ''' if True => is activated
        ''' if false => is deactivated
        ''' </summary>
        ''' <returns></returns>
        Public ReadOnly Property DocWatcherState As Boolean
            Get
                cls_logger.v_logger.Info("DocWatcherState: " & doc_watcher.EnableRaisingEvents.ToString)
                Return doc_watcher.EnableRaisingEvents
            End Get
        End Property

        Public Property cmdDocWatcherOpenPath As New RelayCommand(Sub() DocWatcherOpenPath(), Function() CBool(SettingsCurrent.doc_watcher_use = True AndAlso String.IsNullOrEmpty(SettingsCurrent.doc_watcher_path) = False))

        Private Sub DocWatcherOpenPath()
            With SettingsCurrent
                If .doc_watcher_use = False OrElse String.IsNullOrEmpty(.doc_watcher_path) = True Then Exit Sub
                Process.Start(.doc_watcher_path)
            End With
        End Sub

#End Region
    End Class
End Namespace

