﻿Imports System.Collections.ObjectModel
Imports System.ComponentModel
Imports System.Data.Entity
Imports System.Text
Imports MahApps.Metro.Controls.Dialogs

Namespace HHHB3.ViewModel
    Partial Public Class MainViewModel
        'Part of MVM for Messages
        Private WithEvents __messages As New ObservableCollection(Of message_v01)
        ''' <summary>
        ''' Alle Transaktionstypen
        ''' </summary>
        ''' <returns></returns>
        Public Property MessagesAll As ListCollectionView
        Public Property MessagesUnread As ListCollectionView

        Private Sub MessageLoad()
            __messages.Clear()
            dbo.m_message_v01.Load
            __messages = dbo.m_message_v01.Local
            MessagesAll = New ListCollectionView(__messages)
            MessagesUnread = New ListCollectionView(__messages)
            With MessagesUnread
                .IsLiveFiltering = True
                .LiveFilteringProperties.Add("isread")
                .Filter = New Predicate(Of Object)(AddressOf Filters.MessagesUnread)
                .Refresh()
            End With
            MessagesResetSort(MessagesAll)
            PurgeMessages()
        End Sub

        Private Function MessageAdd(v_newmsg As message_v01) As Boolean
            If v_newmsg Is Nothing Then Return False
            __messages.Add(v_newmsg)
            MessagesResetSort(MessagesAll)
            Return True
        End Function

        Private Sub __messages_CollectionChanged(sender As Object, e As Specialized.NotifyCollectionChangedEventArgs) Handles __messages.CollectionChanged
            If Not MessagesAll Is Nothing Then
                If Not MessagesAll.MoveCurrentToNext Then MessagesAll.MoveCurrentToPrevious()
                MessagesResetSort(MessagesAll)
                Debug.Print(String.Format("All: {0}", MessagesAll.Count))
            End If
            RaisePropertyChanged("MessageAllBackground")
        End Sub

        Public ReadOnly Property MessageAllBackground As SolidColorBrush
            Get
                If MessagesUnread Is Nothing OrElse MessagesUnread.Count = 0 Then
                    Return Nothing
                Else
                    Return Brushes.Red
                End If
            End Get
        End Property

        Private Sub MessagesResetSort(v_list As ListCollectionView)
            If v_list Is Nothing Then Exit Sub
            With v_list
                .SortDescriptions.Clear()
                .SortDescriptions.Add(New SortDescription With {.PropertyName = "isread", .Direction = ListSortDirection.Ascending})
                .SortDescriptions.Add(New SortDescription With {.PropertyName = "priority", .Direction = ListSortDirection.Descending})
                .SortDescriptions.Add(New SortDescription With {.PropertyName = "creation_date", .Direction = ListSortDirection.Descending})
                .Refresh()
            End With
        End Sub

#Region "commands"
        Public Property cmdMessageSetRead As New RelayCommand(Of message_v01)(Sub(p) MessageSetRead(p), Function(p) p IsNot Nothing)
        Public Property cmdMessagesDeleteAll As New RelayCommand(AddressOf MessagesDeleteAll, Function() __messages.Count > 0)
        Public Property cmdMessagesShow As New RelayCommand(AddressOf MessagesShow, Function() __messages.Count > 0)

        Private Sub MessageSetRead(v_msg As message_v01)
            If v_msg Is Nothing Then Exit Sub
            With v_msg
                If .stayalive Then
                    If .isread Then
                        __messages.Remove(v_msg)
                    Else
                        .BeginEdit()
                        .isread = True
                        .EndEdit()
                    End If
                Else
                    If .isread Then
                        'Clear the message:
                        __messages.Remove(v_msg)
                    Else
                        .BeginEdit()
                        .isread = True
                        .EndEdit()
                    End If
                End If
                dbo.SaveChanges()
                MessagesResetSort(MessagesAll)
            End With
            MessagesUnread.Refresh()
            RaisePropertyChanged("MessageAllBackground")
        End Sub

        Private Async Sub MessagesDeleteAll()
            If __messages.Count > 0 Then
                If __messages.Where(Function(x) CBool(x.isread = False)).Count > 0 Then
                    Dim v_strbuilder As New StringBuilder
                    v_strbuilder.AppendLine("Wollen Sie alle ungelesenen Nachrichten ebenfalls löschen?")
                    __messages.Where(Function(x) CBool(x.isread = False)).ForEach(Function(x) v_strbuilder.AppendLine(x.header))
                    Dim dia_settings = New MetroDialogSettings With {.AffirmativeButtonText = "Ja", .NegativeButtonText = "Nein"}

                    Dim v_curwindow As MahApps.Metro.Controls.MetroWindow = Application.Current.Windows.OfType(Of MahApps.Metro.Controls.MetroWindow).SingleOrDefault(Function(x) x.IsActive = True)
                    Dim v_mainwindow As MahApps.Metro.Controls.MetroWindow = DirectCast(Application.Current.MainWindow, MahApps.Metro.Controls.MetroWindow)
                    Dim result As MessageDialogResult

                    If v_curwindow Is Nothing Then
                        result = Await v_mainwindow.ShowMessageAsync("Nachrichten löschen", v_strbuilder.ToString, MessageDialogStyle.AffirmativeAndNegative, dia_settings)
                    Else
                        result = Await v_curwindow.ShowMessageAsync("Nachrichten löschen", v_strbuilder.ToString, MessageDialogStyle.AffirmativeAndNegative, dia_settings)
                    End If
                    If result = MessageDialogResult.Affirmative Then
                        __messages.Clear()
                    Else
                        'Delete only unread:
                        Dim v_read_msg As IList(Of message_v01) = __messages.Where(Function(x) CBool(x.isread)).ToList
                        If v_read_msg.Count > 0 Then
                            For Each v_item As message_v01 In v_read_msg
                                __messages.Remove(v_item)
                            Next
                        End If
                    End If
                Else
                    __messages.Clear()
                End If
            End If
            dbo.SaveChanges()
        End Sub

        ''' <summary>
        ''' Shows all messages in a dialog:
        ''' </summary>
        Private Sub MessagesShow()
            Dim v_dia As New frm_messages
            With v_dia
                .WindowStartupLocation = WindowStartupLocation.CenterScreen
                .ShowDialog()
            End With
        End Sub

#End Region

        ''' <summary>
        ''' Subroutine to purge messages depending on their lifetime.
        ''' Messages with property stayalive true will be ignored.
        ''' </summary>
        Private Sub PurgeMessages()

            Dim v_deathmessages As IList(Of message_v01) = __messages.Where(Function(x) CBool(x.stayalive = False OrElse x.endoflifetime <= Date.Now)).ToList
            If v_deathmessages.Count > 0 Then
                For Each v_item As message_v01 In v_deathmessages
                    __messages.Remove(v_item)
                Next
                dbo.SaveChanges()
            End If
        End Sub

    End Class
End Namespace

