﻿Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.Data.Entity
Imports System.IO
Imports System.Printing
Imports System.Text
Imports LiveCharts
Imports LiveCharts.Wpf
Imports MahApps.Metro.Controls.Dialogs

Namespace HHHB3.ViewModel
    Partial Public Class MainViewModel
        'Part of MVM for reports

        Private WithEvents __reports As New ObservableCollection(Of report_base_v01)
        Public Property Reports As ListCollectionView

        Private Sub ReportsLoad()
            __reports.Clear()
            dbo.m_report_base_v01.Load
            __reports = dbo.m_report_base_v01.Local
            Reports = New ListCollectionView(__reports)
            UpdateReportsAvailable()
        End Sub

        Private Sub __reports_CollectionChanged(sender As Object, e As Specialized.NotifyCollectionChangedEventArgs) Handles __reports.CollectionChanged
            If Not Reports Is Nothing Then
                If Not Reports.MoveCurrentToNext Then Reports.MoveCurrentToPrevious()
            End If
        End Sub

        'List of all available Reports:
        Public Property ReportsAvailable As New List(Of Tuple(Of String, String, String))

        ''' <summary>
        ''' Necessary function to update the available reports if the settings for receiver or persons have changed
        ''' </summary>
        Private Sub UpdateReportsAvailable()
            If ReportsAvailable IsNot Nothing AndAlso ReportsAvailable.Count > 0 Then ReportsAvailable.Clear()

            'Beware: It the Item1 name of the report contains the word top the amount counter will be displayed!
            With ReportsAvailable
                .Add(New Tuple(Of String, String, String)("rep_capital", "Vermögensbericht", "Erstellt einen Bericht über die aktuelle Vermögenssituation"))
                .Add(New Tuple(Of String, String, String)("rep_periods", "Periodenbericht", "Erstellt einen Bericht ähnlich einem Budget mit Einnahmen bzw. Ausgabengegenüberstellung bestimmter Kategorien innerhalb eines Zeitraums aufgeteilt in Zeiträume (Perioden)"))
                .Add(New Tuple(Of String, String, String)("rep_foldertrans", "Ordner Buchungen", "Erstellt einen Bericht der alle Buchungen der einem Ordner zugeordneten Konten beinhaltet"))
                .Add(New Tuple(Of String, String, String)("rep_einaus", "Einnahmen vs. Ausgaben", "Erstellt einen Bericht der die Einnahmen den Ausgaben gegenüberstellt"))
                .Add(New Tuple(Of String, String, String)("rep_cats", "Kategorien", "Erstellt einen Bericht über ausgewählte Kategorien in einem bestimmten Datumsbereich"))
                .Add(New Tuple(Of String, String, String)("rep_topcats", "Top x Kategorien", "Erstellt einen Bericht der die Kategorien mit den meisten Ausgaben oder Einnahmen in einer definierten Anzahl und Reihenfolge anzeigt"))
                If SettingsCurrent.active_receiver Then
                    .Add(New Tuple(Of String, String, String)("rep_receiver", "Empfänger", "Erstellt einen Bericht über ausgewählte Empfänger in einem bestimmten Datumsbereich"))
                    .Add(New Tuple(Of String, String, String)("rep_topreceiver", "Top x Empfänger", "Erstellt einen Bericht der die Empfänger mit den meisten Ausgaben oder Einnahmen in einer definierten Anzahl und Reihenfolge anzeigt"))
                End If
                If SettingsCurrent.active_person Then
                    .Add(New Tuple(Of String, String, String)("rep_person", "Personen", "Erstellt einen Bericht über ausgewählte Personen in einem bestimmten Datumsbereich"))
                    .Add(New Tuple(Of String, String, String)("rep_topperson", "Top x Personen", "Erstellt einen Bericht der die Personen mit den meisten Ausgaben oder Einnahmen in einer definierten Anzahl und Reihenfolge anzeigt"))
                End If
            End With
        End Sub

#Region "Commands"

        Public Property cmdReportAdd As New RelayCommand(AddressOf ReportAdd)
        Public Property cmdReportDelete As New RelayCommand(Of report_base_v01)(Sub(p) ReportDelete(p), Function(p) p IsNot Nothing)
        Public Property cmdReportDo As New RelayCommand(Of report_base_v01)(Sub(p) ReportDo(p), Function(p) p IsNot Nothing)
        Public Property cmdReportPrint As New RelayCommand(Of report_base_v01)(Sub(p) ReportPrint(p), Function(p) ReportPrintCanExecute(p))
        Public Property cmdReportSave As New RelayCommand(Of report_base_v01)(Sub(p) ReportSave(p), Function(p) ReportSaveCanExecute(p))

        Private Sub ReportAdd()
            Dim v_dia As New frm_report_add With {.WindowStartupLocation = WindowStartupLocation.CenterScreen}
            Dim v_new_report As New report_base_v01
            With v_dia
                .DataContext = v_new_report
                .ShowDialog()
                If .DialogResult Then
                    Reports.AddNewItem(v_new_report)
                    Reports.CommitNew()
                    dbo.SaveChanges()
                Else
                    v_new_report = Nothing
                End If
            End With
        End Sub

        Private Async Sub ReportDelete(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub

            Dim dia_settings = New MetroDialogSettings With {.AffirmativeButtonText = "Ja", .NegativeButtonText = "Nein"}
            Dim v_mainwindow As MahApps.Metro.Controls.MetroWindow = DirectCast(Application.Current.MainWindow, MahApps.Metro.Controls.MetroWindow)
            Dim result As MessageDialogResult = Await v_mainwindow.ShowMessageAsync("Bericht löschen", String.Format("Wollen Sie den Bericht {0} wirklich löschen?", v_rep.rep_bezeichnung), MessageDialogStyle.AffirmativeAndNegative, dia_settings)
            If result = MessageDialogResult.Affirmative Then
                Reports.Remove(v_rep)
                dbo.SaveChanges()
            End If
        End Sub

        Private Sub ReportDo(v_rep As report_base_v01)
            GlobalBusy = True
            If v_rep Is Nothing Then Exit Sub
            'Depending on the type of the report decide what to do:
            Select Case v_rep.type
                Case "rep_capital"
                    RepCapitalDo(v_rep)
                Case "rep_foldertrans"
                    RepFoldertransDo(v_rep)
                Case "rep_einaus"
                    RepEinausDo(v_rep)
                Case "rep_cats"
                    RepCatsDo(v_rep)
                Case "rep_topcats"
                    RepTopCatsDo(v_rep)
                Case "rep_receiver"
                    RepReceiverDo(v_rep)
                Case "rep_topreceiver"
                    RepTopReceiverDo(v_rep)
                Case "rep_person"
                    RepPersonDo(v_rep)
                Case "rep_topperson"
                    RepTopPersonDo(v_rep)
                Case "rep_periods"
                    RepPeriodsDo(v_rep)
                Case Else
                    cls_logger.v_logger.Error("Not able to retrieve Report Typ at ReportDo")
            End Select
            GlobalBusy = False
        End Sub

        Private Sub RepCapitalDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            If String.IsNullOrEmpty(v_rep.selected_konto_ids) Then Exit Sub

            With v_rep
                .clear_report()

                Dim v_columns As New ColumnSeries With {.Title = "Konto Wert"}
                Dim v_values As New ChartValues(Of Double)

                'Retrieve the selected kontos:
                Dim v_sel_kontos As String() = v_rep.selected_konto_ids.Split(";")
                For i As Integer = 0 To v_sel_kontos.Count - 1
                    Dim v_itemc As Integer = i
                    Dim v_kto As konto_v01 = __konten.Where(Function(x) x.kto_id = CInt(v_sel_kontos(v_itemc))).FirstOrDefault
                    If v_kto IsNot Nothing Then
                        .ResultCapital.Add(New Tuple(Of String, String, Decimal)(v_kto.nv_folder_2_konto_back.bezeichnung, v_kto.bezeichnung, v_kto.TotalamountTimeFrame(v_rep.date_start, v_rep.date_end, v_rep.positive)))
                    End If
                Next

                For Each v_item As Tuple(Of String, String, Decimal) In .ResultCapital
                    .Chart_Labels.Add(v_item.Item1 & "-" & v_item.Item2)
                    v_values.Add(v_item.Item3)
                    .ResultCapitalEndsum += v_item.Item3
                Next

                v_columns.Values = v_values
                .Chart_Series.Add(v_columns)
            End With
        End Sub

        Private Sub RepFoldertransDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            If String.IsNullOrEmpty(v_rep.selected_folder_ids) Then Exit Sub

            With v_rep
                .clear_report()

                Dim v_columns As New ColumnSeries With {.Title = "Ordner Wert"}
                Dim v_values As New ChartValues(Of Double)

                'Retrieve the selected kontos:
                Dim v_sel_folders As String() = v_rep.selected_folder_ids.Split(";")
                For i As Integer = 0 To v_sel_folders.Count - 1
                    Dim v_itemc As Integer = i
                    Dim v_folder As folder_v01 = __folders.Where(Function(x) x.fol_id = CInt(v_sel_folders(v_itemc))).FirstOrDefault
                    If v_folder IsNot Nothing Then
                        .ResultFolderTrans.Add(New Tuple(Of String, Decimal)(v_folder.bezeichnung, v_folder.TotalamountTimeFrame(v_rep.date_start, v_rep.date_end)))
                    End If
                Next

                For Each v_item As Tuple(Of String, Decimal) In .ResultFolderTrans
                    .Chart_Labels.Add(v_item.Item1)
                    v_values.Add(v_item.Item2)
                    .ResultCapitalEndsum += v_item.Item2
                Next

                v_columns.Values = v_values
                .Chart_Series.Add(v_columns)
            End With

        End Sub

        Private Sub RepEinausDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            With v_rep
                .clear_report()

                Dim v_values_ein As New ChartValues(Of Double)
                Dim v_values_aus As New ChartValues(Of Double)

                Dim v_posvalues As Decimal = 0
                Dim v_negvalues As Decimal = 0
                v_posvalues = __transactions.Where(Function(x) x.datum <= .date_end AndAlso x.datum >= .date_start AndAlso x.real_totamount > 0).Sum(Function(x) x.real_totamount)
                v_negvalues = __transactions.Where(Function(x) x.datum <= .date_end AndAlso x.datum >= .date_start AndAlso x.real_totamount < 0).Sum(Function(x) x.real_totamount)

                v_values_ein.Add(v_posvalues)
                v_values_aus.Add(v_negvalues)
                Dim v_columns_ein As New ColumnSeries With {.Title = "Einnahmen", .Values = v_values_ein, .Fill = SettingsCurrent.color_pos_value_brush}
                Dim v_columns_aus As New ColumnSeries With {.Title = "Ausgaben", .Values = v_values_aus, .Fill = SettingsCurrent.color_neg_value_brush}
                .Chart_Series.Add(v_columns_ein)
                .Chart_Series.Add(v_columns_aus)

                .ResultFolderTrans.Add(New Tuple(Of String, Decimal)("Einnahmen", v_posvalues))
                .ResultFolderTrans.Add(New Tuple(Of String, Decimal)("Ausgaben", v_negvalues))

            End With
        End Sub

        Private Sub RepCatsDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            With v_rep
                .clear_report()
                Dim v_columns As New ColumnSeries With {.Title = "Kategorie Wert", .Fill = If(v_rep.positive, SettingsCurrent.color_pos_value_brush, SettingsCurrent.color_neg_value_brush)}
                Dim v_values As New ChartValues(Of Double)

                'Retrieve the selected Categories:
                Dim v_sel_cats As String() = v_rep.selected_category_ids.Split(";")
                For i As Integer = 0 To v_sel_cats.Count - 1
                    Dim v_itemc As Integer = i
                    Dim v_cat As category_v01 = __categories.Where(Function(x) x.cat_id = CInt(v_sel_cats(v_itemc))).FirstOrDefault
                    If v_cat IsNot Nothing Then
                        .ResultCapital.Add(New Tuple(Of String, String, Decimal)(If(v_cat.nv_subcats_back Is Nothing, "", v_cat.nv_subcats_back.bezeichnung), v_cat.bezeichnung, v_cat.totamount_timespan_caluclator(v_rep.date_start, v_rep.date_end)))
                    End If
                Next
                For Each v_item As Tuple(Of String, String, Decimal) In .ResultCapital
                    .Chart_Labels.Add(v_item.Item2)
                    v_values.Add(v_item.Item3)
                    .ResultCapitalEndsum += v_item.Item3
                    'Piechart:
                    Dim v_newpie As New PieSeries With {.Title = v_item.Item2}
                    v_newpie.Values = New ChartValues(Of Double) From {v_item.Item3}
                    .PieChart_Series.Add(v_newpie)
                Next
                v_columns.Values = v_values
                .Chart_Series.Add(v_columns)
            End With
        End Sub

        Private Sub RepTopCatsDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            If v_rep.count = Nothing OrElse v_rep.count = 0 Then Exit Sub

            With v_rep
                    .clear_report()
                Dim v_columns As New ColumnSeries With {.Title = "Kategorie Wert", .Fill = If(v_rep.positive, SettingsCurrent.color_pos_value_brush, SettingsCurrent.color_neg_value_brush)}
                Dim v_values As New ChartValues(Of Double)

                'Retrieve the all Categories:
                Dim v_active_cats As ListCollectionView = New ListCollectionView(__categories) With {.Filter = New Predicate(Of Object)(AddressOf Filters.CatsAktiv)}

                'For each categpry calculate the amount in the timeframe:
                For Each v_cat As category_v01 In v_active_cats
                    v_cat.totamount_timespan_caluclator(v_rep.date_start, v_rep.date_end)
                Next

                'Apply a filter to remove all 0 amounts:
                If v_rep.positive = True Then
                    v_active_cats.Filter = New Predicate(Of Object)(AddressOf Filters.TotAmountTimespanBiggerZero)
                    'After the value is set we can do the sorting:
                    v_active_cats.SortDescriptions.Add(New ComponentModel.SortDescription With {.PropertyName = "totamount_timespan", .Direction = ComponentModel.ListSortDirection.Descending})
                Else
                    v_active_cats.Filter = New Predicate(Of Object)(AddressOf Filters.TotAmountTimespanSmallerZero)
                    'After the value is set we can do the sorting:
                    v_active_cats.SortDescriptions.Add(New ComponentModel.SortDescription With {.PropertyName = "totamount_timespan", .Direction = ComponentModel.ListSortDirection.Ascending})
                End If

                v_active_cats.Refresh()
                v_active_cats.MoveCurrentToFirst()
                For i As Integer = 0 To v_rep.count - 1
                    If v_active_cats.IsCurrentBeforeFirst Then Exit For
                    Dim v_cat As category_v01 = CType(v_active_cats.CurrentItem, category_v01)
                    .ResultCapital.Add(New Tuple(Of String, String, Decimal)(If(v_cat.nv_subcats_back Is Nothing, "", v_cat.nv_subcats_back.bezeichnung), v_cat.bezeichnung, v_cat.totamount_timespan_caluclator(v_rep.date_start, v_rep.date_end)))
                    v_active_cats.MoveCurrentToNext()
                    If v_active_cats.IsCurrentAfterLast Then Exit For
                Next
                v_active_cats = Nothing

                For Each v_item As Tuple(Of String, String, Decimal) In .ResultCapital
                    .Chart_Labels.Add(v_item.Item2)
                    v_values.Add(v_item.Item3)
                    .ResultCapitalEndsum += v_item.Item3
                    'Piechart:
                    Dim v_newpie As New PieSeries With {.Title = v_item.Item2}
                    v_newpie.Values = New ChartValues(Of Double) From {v_item.Item3}
                    .PieChart_Series.Add(v_newpie)
                Next
                v_columns.Values = v_values
                .Chart_Series.Add(v_columns)
            End With
        End Sub

        Private Sub RepReceiverDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            With v_rep
                .clear_report()
                Dim v_columns As New ColumnSeries With {.Title = "Empfänger Wert", .Fill = If(v_rep.positive, SettingsCurrent.color_pos_value_brush, SettingsCurrent.color_neg_value_brush)}
                Dim v_values As New ChartValues(Of Double)

                'Retrieve the selected Categories:
                Dim v_sel_receiver As String() = v_rep.selected_receiver_ids.Split(";")
                For i As Integer = 0 To v_sel_receiver.Count - 1
                    Dim v_itemc As Integer = i
                    Dim v_rec As receiver_v01 = __reveicer.Where(Function(x) x.rec_id = CInt(v_sel_receiver(v_itemc))).FirstOrDefault
                    If v_rec IsNot Nothing Then
                        .ResultFolderTrans.Add(New Tuple(Of String, Decimal)(v_rec.bezeichnung, v_rec.totamount_timespan_caluclator(v_rep.date_start, v_rep.date_end)))
                    End If
                Next
                For Each v_item As Tuple(Of String, Decimal) In .ResultFolderTrans
                    .Chart_Labels.Add(v_item.Item1)
                    v_values.Add(v_item.Item2)
                    .ResultCapitalEndsum += v_item.Item2
                    'Piechart:
                    Dim v_newpie As New PieSeries With {.Title = v_item.Item1}
                    v_newpie.Values = New ChartValues(Of Double) From {v_item.Item2}
                    .PieChart_Series.Add(v_newpie)
                Next
                v_columns.Values = v_values
                .Chart_Series.Add(v_columns)
            End With
        End Sub

        Private Sub RepTopReceiverDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            If v_rep.count = Nothing OrElse v_rep.count = 0 Then Exit Sub

            With v_rep
                .clear_report()
                Dim v_columns As New ColumnSeries With {.Title = "Empfänger Wert", .Fill = If(v_rep.positive, SettingsCurrent.color_pos_value_brush, SettingsCurrent.color_neg_value_brush)}
                Dim v_values As New ChartValues(Of Double)

                'Retrieve the all Categories:
                Dim v_active_rec As ListCollectionView = New ListCollectionView(__reveicer) With {.Filter = New Predicate(Of Object)(AddressOf Filters.ObjAktiv)}

                'For each categpry calculate the amount in the timeframe:
                For Each v_rec As receiver_v01 In v_active_rec
                    v_rec.totamount_timespan_caluclator(v_rep.date_start, v_rep.date_end)
                    Debug.Print(String.Format("Calculated: {0} with amount {1}", v_rec.bezeichnung, v_rec.totamount_timespan))
                Next

                'Apply a filter to remove all 0 amounts:
                If v_rep.positive = True Then
                    v_active_rec.Filter = New Predicate(Of Object)(AddressOf Filters.TotAmountTimespanBiggerZero)
                    'After the value is set we can do the sorting:
                    v_active_rec.SortDescriptions.Add(New ComponentModel.SortDescription With {.PropertyName = "totamount_timespan", .Direction = ComponentModel.ListSortDirection.Descending})
                Else
                    v_active_rec.Filter = New Predicate(Of Object)(AddressOf Filters.TotAmountTimespanSmallerZero)
                    'After the value is set we can do the sorting:
                    v_active_rec.SortDescriptions.Add(New ComponentModel.SortDescription With {.PropertyName = "totamount_timespan", .Direction = ComponentModel.ListSortDirection.Ascending})
                End If

                For Each v_rec As receiver_v01 In v_active_rec
                    Debug.Print(String.Format("Calculated after filter and sort: {0} with amount {1}", v_rec.bezeichnung, v_rec.totamount_timespan))
                Next

                v_active_rec.Refresh()
                v_active_rec.MoveCurrentToFirst()
                For i As Integer = 0 To v_rep.count - 1
                    If v_active_rec.IsCurrentBeforeFirst Then Exit For
                    Dim v_rec As receiver_v01 = CType(v_active_rec.CurrentItem, receiver_v01)
                    .ResultFolderTrans.Add(New Tuple(Of String, Decimal)(v_rec.bezeichnung, v_rec.totamount_timespan_caluclator(v_rep.date_start, v_rep.date_end)))
                    v_active_rec.MoveCurrentToNext()
                    If v_active_rec.IsCurrentAfterLast Then Exit For
                Next
                v_active_rec = Nothing

                For Each v_item As Tuple(Of String, Decimal) In .ResultFolderTrans
                    .Chart_Labels.Add(v_item.Item1)
                    v_values.Add(v_item.Item2)
                    .ResultCapitalEndsum += v_item.Item2
                    'Piechart:
                    Dim v_newpie As New PieSeries With {.Title = v_item.Item1}
                    v_newpie.Values = New ChartValues(Of Double) From {v_item.Item2}
                    .PieChart_Series.Add(v_newpie)
                Next
                v_columns.Values = v_values
                .Chart_Series.Add(v_columns)
            End With
        End Sub

        Private Sub RepPersonDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            With v_rep
                .clear_report()
                Dim v_columns As New ColumnSeries With {.Title = "Personen Wert", .Fill = If(v_rep.positive, SettingsCurrent.color_pos_value_brush, SettingsCurrent.color_neg_value_brush)}
                Dim v_values As New ChartValues(Of Double)

                'Retrieve the selected Categories:
                Dim v_sel_person As String() = v_rep.selected_person_ids.Split(";")
                For i As Integer = 0 To v_sel_person.Count - 1
                    Dim v_itemc As Integer = i
                    Dim v_pers As person_v01 = __persons.Where(Function(x) x.pers_id = CInt(v_sel_person(v_itemc))).FirstOrDefault
                    If v_pers IsNot Nothing Then
                        .ResultFolderTrans.Add(New Tuple(Of String, Decimal)(v_pers.bezeichnung, v_pers.totamount_timespan_caluclator(v_rep.date_start, v_rep.date_end, If(v_rep.positive, CShort(1), CShort(-1)))))
                    End If
                Next
                For Each v_item As Tuple(Of String, Decimal) In .ResultFolderTrans
                    .Chart_Labels.Add(v_item.Item1)
                    v_values.Add(v_item.Item2)
                    .ResultCapitalEndsum += v_item.Item2
                    'Piechart:
                    Dim v_newpie As New PieSeries With {.Title = v_item.Item1}
                    v_newpie.Values = New ChartValues(Of Double) From {v_item.Item2}
                    .PieChart_Series.Add(v_newpie)
                Next
                v_columns.Values = v_values
                .Chart_Series.Add(v_columns)
            End With
        End Sub

        Private Sub RepTopPersonDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            If v_rep.count = Nothing OrElse v_rep.count = 0 Then Exit Sub

            With v_rep
                .clear_report()
                Dim v_columns As New ColumnSeries With {.Title = "Person Wert", .Fill = If(v_rep.positive, SettingsCurrent.color_pos_value_brush, SettingsCurrent.color_neg_value_brush)}
                Dim v_values As New ChartValues(Of Double)

                'Retrieve the all Categories:
                Dim v_active_pers As ListCollectionView = New ListCollectionView(__persons) With {.Filter = New Predicate(Of Object)(AddressOf Filters.ObjAktiv)}

                'For each categpry calculate the amount in the timeframe:
                For Each v_pers As person_v01 In v_active_pers
                    v_pers.totamount_timespan_caluclator(v_rep.date_start, v_rep.date_end, If(v_rep.positive, CShort(1), CShort(-1)))
                    'Debug.Print(String.Format("Calculated: {0} with amount {1}", v_pers.bezeichnung, v_pers.totamount_timespan))
                Next

                'Apply a filter to remove all 0 amounts:
                If v_rep.positive = True Then
                    v_active_pers.Filter = New Predicate(Of Object)(AddressOf Filters.TotAmountTimespanBiggerZero)
                    'After the value is set we can do the sorting:
                    v_active_pers.SortDescriptions.Add(New ComponentModel.SortDescription With {.PropertyName = "totamount_timespan", .Direction = ComponentModel.ListSortDirection.Descending})
                Else
                    v_active_pers.Filter = New Predicate(Of Object)(AddressOf Filters.TotAmountTimespanSmallerZero)
                    'After the value is set we can do the sorting:
                    v_active_pers.SortDescriptions.Add(New ComponentModel.SortDescription With {.PropertyName = "totamount_timespan", .Direction = ComponentModel.ListSortDirection.Ascending})
                End If
                v_active_pers.Refresh()

                'For Each v_pers As person_v01 In v_active_pers
                '    Debug.Print(String.Format("Calculated after filter and sort: {0} with amount {1}", v_pers.bezeichnung, v_pers.totamount_timespan))
                'Next

                v_active_pers.MoveCurrentToFirst()

                For i As Integer = 0 To v_rep.count - 1
                    If v_active_pers.IsCurrentBeforeFirst Then Exit For
                    Dim v_pers As person_v01 = CType(v_active_pers.CurrentItem, person_v01)
                    .ResultFolderTrans.Add(New Tuple(Of String, Decimal)(v_pers.bezeichnung, v_pers.totamount_timespan))
                    v_active_pers.MoveCurrentToNext()
                    If v_active_pers.IsCurrentAfterLast Then Exit For
                Next
                v_active_pers = Nothing

                For Each v_item As Tuple(Of String, Decimal) In .ResultFolderTrans
                    .Chart_Labels.Add(v_item.Item1)
                    v_values.Add(v_item.Item2)
                    .ResultCapitalEndsum += v_item.Item2
                    'Piechart:
                    Dim v_newpie As New PieSeries With {.Title = v_item.Item1}
                    v_newpie.Values = New ChartValues(Of Double) From {v_item.Item2}
                    .PieChart_Series.Add(v_newpie)
                Next
                v_columns.Values = v_values
                .Chart_Series.Add(v_columns)
            End With
        End Sub

        Private Sub RepPeriodsDo(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            'The Period Report need:
            'a Start date
            'a defined period
            'a amount of periods
            'some income and expend categories
            'and the info if the categories should be summarized if they are parents. => stored in the positive property => if true summarize if false then not
            With v_rep
                If .date_start = Nothing Then Exit Sub
                If .selected_period_period = Nothing Then Exit Sub
                If .count = Nothing Then Exit Sub
                If .selected_category_ids Is Nothing Then Exit Sub
                If .selected_period_negative_category_ids Is Nothing Then Exit Sub

                .clear_report()

                'First: Split all the selectedcategories as a real category:
                If ReportPeriodsCategorySplitter(.ResultPeriodsPosCats, .selected_category_ids, True) = False OrElse ReportPeriodsCategorySplitter(.ResultPeriodsNegCats, .selected_period_negative_category_ids, False) = False Then
                    cls_logger.v_logger.Error("Report Periods failed to split categories")
                    MessageBox.Show("Report Periods failed to split categories", "Fehler", MessageBoxButton.OK, MessageBoxImage.Error)
                    Exit Sub
                End If

                'Second calculate the periods:
                Dim v_template_period_list As New List(Of cls_repperiod_perioditem)
                Dim v_selected_timeframe As cls_timeframe = __timeframes_periods_report.FirstOrDefault(Function(x) x.Id = .selected_period_period)
                If v_selected_timeframe Is Nothing Then Exit Sub
                Dim v_next_date As Date = .date_start

                For i As Integer = 1 To .count
                    Dim v_end_date As Date = v_next_date
                    Dim v_period_start_date As Date = v_next_date
                    Select Case v_selected_timeframe.scope
                        Case "D"
                            'Scope is daily
                            v_end_date = v_next_date.AddDays(v_selected_timeframe.count)
                            v_next_date = v_end_date.AddDays(1)
                        Case "M"
                            'Scope is monthly
                            v_end_date = v_next_date.AddMonths(v_selected_timeframe.count).AddDays(-1)
                            v_next_date = v_end_date.AddDays(1)
                        Case "Y"
                            'Scope is yearly
                            v_end_date = v_next_date.AddYears(v_selected_timeframe.count)
                            v_next_date = v_end_date.AddYears(1)
                    End Select
                    v_template_period_list.Add(New cls_repperiod_perioditem(String.Format("{0} {1}", v_selected_timeframe.bezeichnung, i.ToString), v_period_start_date, v_end_date, 0))
                    'The header holds the name, the datespan, the positive totamount and the negative totamount of a period
                    .ResultPeriodsHeader.Add(New cls_RepPeriodsHeaderItem(String.Format("{0} {1}", v_selected_timeframe.bezeichnung, i.ToString), String.Format("{0} - {1}", v_period_start_date.ToShortDateString, v_end_date.ToShortDateString), 0, 0))
                Next

                'Now all Possible Periods are stored in a template list add the entire list to all the categories:
                If ReportPeriodsAddPeriods(.ResultPeriodsPosCats, v_template_period_list) = False OrElse ReportPeriodsAddPeriods(.ResultPeriodsNegCats, v_template_period_list) = False Then
                    cls_logger.v_logger.Error("Report Periods failed to add periods in Categories")
                    MessageBox.Show("Report Periods failed to add periods in Categories", "Fehler", MessageBoxButton.OK, MessageBoxImage.Error)
                    Exit Sub
                End If

                If ReportPeriodsCalculateValues(.ResultPeriodsPosCats, .positive) = False OrElse ReportPeriodsCalculateValues(.ResultPeriodsNegCats, .positive) = False Then
                    cls_logger.v_logger.Error("Report Periods failed to calculate values of periods")
                    MessageBox.Show("Report Periods failed to calculate values of periods", "Fehler", MessageBoxButton.OK, MessageBoxImage.Error)
                    Exit Sub
                End If
                'Now calculate the total positive Amount:
                .ResultPeriodsPosAmount = .ResultPeriodsPosCats.Sum(Function(x) x.TotAmount)
                'Now calculate the total negative amount:
                .ResultPeriodsNegAmount = .ResultPeriodsNegCats.Sum(Function(x) x.TotAmount)

                'Now Calculate the Pos and Negative TotalValue of each Period:
                For Each v_headeritem As cls_RepPeriodsHeaderItem In .ResultPeriodsHeader
                    v_headeritem.PosValue = .ResultPeriodsPosCats.Sum(Function(x) x.PeriodTotAmount(v_headeritem.Bezeichnung))
                    v_headeritem.NegValue = .ResultPeriodsNegCats.Sum(Function(x) x.PeriodTotAmount(v_headeritem.Bezeichnung))
                Next

                'Now make the diagramm:
                Dim v_columns_pos As New ColumnSeries With {.Title = "Einnahmen", .Fill = SettingsCurrent.color_pos_value_brush}
                Dim v_columns_neg As New ColumnSeries With {.Title = "Ausgaben", .Fill = SettingsCurrent.color_neg_value_brush}
                Dim v_values_pos As New ChartValues(Of Decimal)
                Dim v_values_neg As New ChartValues(Of Decimal)

                For Each v_item As cls_RepPeriodsHeaderItem In .ResultPeriodsHeader
                    .Chart_Labels.Add(v_item.Bezeichnung)
                    v_values_pos.Add(v_item.PosValue)
                    v_values_neg.Add(v_item.NegValue * -1)
                Next
                v_columns_pos.Values = v_values_pos
                v_columns_neg.Values = v_values_neg
                .Chart_Series.Add(v_columns_pos)
                .Chart_Series.Add(v_columns_neg)
            End With
        End Sub

        Private Function ReportPeriodsCategorySplitter(v_categorylist As ObservableCollection(Of cls_RepPeriodsCategory), v_sel_cats As String, v_isincome As Boolean) As Boolean
            If v_categorylist Is Nothing OrElse String.IsNullOrEmpty(v_sel_cats) Then Return False
            Dim v_cat_ids As String() = v_sel_cats.Split(";")
            For i As Integer = 0 To v_cat_ids.Count - 1
                Dim v_itemc As Integer = i
                Dim v_cat As category_v01 = __categories.Where(Function(x) x.cat_id = CInt(v_cat_ids(v_itemc))).FirstOrDefault
                If Not v_cat Is Nothing Then
                    v_categorylist.Add(New cls_RepPeriodsCategory(v_cat.fullname, v_cat.cat_id, v_isincome))
                End If
            Next
            v_cat_ids = Nothing
            Return True
        End Function

        Private Function ReportPeriodsAddPeriods(v_categorylist As ObservableCollection(Of cls_RepPeriodsCategory), v_periods As List(Of cls_repperiod_perioditem)) As Boolean
            If v_categorylist Is Nothing OrElse v_periods Is Nothing Then Return False

            For Each v_item As cls_RepPeriodsCategory In v_categorylist
                For Each v_period As cls_repperiod_perioditem In v_periods
                    v_item.Periods.Add(New cls_repperiod_perioditem(v_period.Bezeichnung, v_period.StartDate, v_period.EndDate, 0))
                Next
            Next
            Return True
        End Function

        Private Function ReportPeriodsCalculateValues(v_categorylist As ObservableCollection(Of cls_RepPeriodsCategory), v_summarize_parents As Boolean) As Boolean
            If v_categorylist Is Nothing Then Return False
            For Each v_repcategory As cls_RepPeriodsCategory In v_categorylist
                'Get the real Category:
                Dim v_cat As category_v01 = __categories.Where(Function(x) x.cat_id = v_repcategory.CatId).FirstOrDefault
                If v_cat Is Nothing Then Continue For
                For Each v_period As cls_repperiod_perioditem In v_repcategory.Periods
                    'Now do the calculation stuff:
                    v_period.Amount = v_cat.nv_cat_trans2kat.Where(Function(x) x.nv_trans_t2k_back.nv_transtyp_trans_back.multiplier <> 0 AndAlso x.nv_trans_t2k_back.datum >= v_period.StartDate AndAlso x.nv_trans_t2k_back.datum <= v_period.EndDate).Sum(Function(x) x.amount)
                    If v_repcategory.isIncome = False Then v_period.Amount = v_period.Amount * -1
                Next
                'Check if we should aggregate the parent categories:
                If v_summarize_parents Then
                    If v_cat.nv_subcats IsNot Nothing OrElse v_cat.nv_subcats.Count > 0 Then
                        For Each v_subcat As category_v01 In v_cat.nv_subcats
                            For Each v_period As cls_repperiod_perioditem In v_repcategory.Periods
                                'Now do the calculation stuff again for the childs:
                                Dim v_temp_amount As Decimal = v_subcat.nv_cat_trans2kat.Where(Function(x) x.nv_trans_t2k_back.nv_transtyp_trans_back.multiplier <> 0 AndAlso x.nv_trans_t2k_back.datum >= v_period.StartDate AndAlso x.nv_trans_t2k_back.datum <= v_period.EndDate).Sum(Function(x) x.amount)
                                If v_repcategory.isIncome = False Then v_temp_amount = v_temp_amount * -1
                                v_period.Amount += v_temp_amount
                                v_temp_amount = Nothing
                            Next
                        Next
                    End If
                End If
                v_cat = Nothing
                'Now calculate the total amount of this PeriodCategory:
                v_repcategory.TotAmount = v_repcategory.Periods.Sum(Function(x) x.Amount)
            Next
            Return True
        End Function


#Region "Print"
        Private Sub ReportPrint(v_rep As report_base_v01)
            If v_rep Is Nothing Then Exit Sub
            Dim v_result As Boolean = False
            'Depending on the type of the report decide what to do:
            Select Case v_rep.type
                Case "rep_capital"
                    v_result = RepCapitalPrintPrepare(v_rep)
                Case "rep_foldertrans"
                    v_result = RepFoldertransEinAusPrintPrepare(v_rep, "Ordner")
                Case "rep_einaus"
                    v_result = RepFoldertransEinAusPrintPrepare(v_rep, "Ein/Aus")
                Case "rep_cats"
                    v_result = RepCatsTopCatsPrintPrepare(v_rep, "Kategorien", False)
                Case "rep_topcats"
                    v_result = RepCatsTopCatsPrintPrepare(v_rep, "Top-Kategorien", True)
                Case "rep_receiver"
                    v_result = RepReceiverPersonTopReceiverTopPersonPrintPrepare(v_rep, "Empfänger", False)
                Case "rep_topreceiver"
                    v_result = RepReceiverPersonTopReceiverTopPersonPrintPrepare(v_rep, "Top-Empfänger", True)
                Case "rep_person"
                    v_result = RepReceiverPersonTopReceiverTopPersonPrintPrepare(v_rep, "Personen", False)
                Case "rep_topperson"
                    v_result = RepReceiverPersonTopReceiverTopPersonPrintPrepare(v_rep, "Top-Personen", True)
                Case "rep_periods"
                    'Periods Print not ready yet
                    v_result = RepPeriodsPrintPrepare(v_rep, "Perioden", True)
                Case Else
                    v_result = False
                    cls_logger.v_logger.Error("Not able to retrieve Report Typ at ReportPrint")
            End Select

            If v_result Then
                'Go to Print
                If v_rep.type = "rep_periods" Then
                    ReportDoPrint(v_rep, False)
                Else
                    ReportDoPrint(v_rep, True)
                End If
            Else
                cls_logger.v_logger.Error(String.Format("Print preparation for Report Type {0} failed", v_rep.type))
                Dim vMainWindow As MainWindow = CType(Application.Current.MainWindow, MainWindow)
                vMainWindow.ShowMessageAsync("Fehler", "Fehler bei Druckvorbereitung, Report konnte nicht gedruckt werden", MessageDialogStyle.Affirmative)
            End If


        End Sub

        Private Function ReportPrintCanExecute(v_rep As report_base_v01) As Boolean
            If v_rep Is Nothing Then Return False
            If String.IsNullOrEmpty(v_rep.type) Then Return False
            With v_rep
                'Depending on the type of the report decide what to do:
                Select Case v_rep.type
                    Case "rep_capital"
                        If .ResultCapital Is Nothing OrElse .ResultCapital.Count = 0 Then Return False
                    Case "rep_foldertrans"
                        If .ResultFolderTrans Is Nothing OrElse .ResultFolderTrans.Count = 0 Then Return False
                    Case "rep_einaus"
                        If .ResultFolderTrans Is Nothing OrElse .ResultFolderTrans.Count = 0 Then Return False
                    Case "rep_cats"
                        If .ResultCapital Is Nothing OrElse .ResultCapital.Count = 0 Then Return False
                    Case "rep_topcats"
                        If .ResultCapital Is Nothing OrElse .ResultCapital.Count = 0 Then Return False
                    Case "rep_receiver"
                        If .ResultFolderTrans Is Nothing OrElse .ResultFolderTrans.Count = 0 Then Return False
                    Case "rep_topreceiver"
                        If .ResultFolderTrans Is Nothing OrElse .ResultFolderTrans.Count = 0 Then Return False
                    Case "rep_person"
                        If .ResultFolderTrans Is Nothing OrElse .ResultFolderTrans.Count = 0 Then Return False
                    Case "rep_topperson"
                        If .ResultFolderTrans Is Nothing OrElse .ResultFolderTrans.Count = 0 Then Return False
                    Case "rep_periods"
                        If .ResultPeriodsHeader Is Nothing OrElse .ResultPeriodsPosCats Is Nothing OrElse .ResultPeriodsNegCats Is Nothing OrElse .ResultPeriodsHeader.Count = 0 Then Return False
                    Case Else
                        Return False
                End Select
                Return True
            End With
        End Function

        Private Function RepCapitalPrintPrepare(v_rep As report_base_v01) As Boolean
            If v_rep Is Nothing Then Return False
            With v_rep
                If .ResultCapital Is Nothing OrElse .ResultCapital.Count = 0 Then Return False

                Dim v_rowcount As Integer = 1
                Dim v_oddrowbrush As Brush = SettingsCurrent.color_background_oddrow_brush
                Dim v_justrowbrush As Brush = SettingsCurrent.color_background_justrow_brush
                Dim v_headerbackgroundbrush As Brush = SettingsCurrent.color_background_header_brush
                Dim v_header_fontsize As Double = SettingsCurrent.header_fontsize - 10
                Dim v_row_fontsize As Double = SettingsCurrent.row_fontsize - 4

                'Create the report header:
                .PrintDocument.Blocks.Clear()

                'Headers for the Konto Table:
                Dim v_header As New Dictionary(Of String, Integer) From {
                    New KeyValuePair(Of String, Integer)("Ordner", 20),
                    New KeyValuePair(Of String, Integer)("Konto", 20),
                    New KeyValuePair(Of String, Integer)("Betrag", 20)}

                'Define the report:
                With .PrintDocument
                    .Blocks.Clear()
                    'Header:
                    .Blocks.Add(New Paragraph(New Run(String.Format("Vermögensbericht - {0}", v_rep.rep_bezeichnung))) With {.FontSize = SettingsCurrent.header_fontsize})
                    .Blocks.Add(New Paragraph(New Run(Date.Now.ToShortDateString)) With {.FontSize = 12, .TextAlignment = TextAlignment.Right})
                    .Blocks.Add(New Paragraph(New Run("Kriterien")))
                    .Blocks.Add(New Paragraph(New Run(String.Format("Datumsbereich von {0} bis {1}", v_rep.date_start.ToShortDateString, v_rep.date_end.ToShortDateString))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})

                    Dim v_kontos As New StringBuilder
                    v_kontos.Append("Berücksichtigte Konten:")
                    For Each v_item As Tuple(Of String, String, Decimal) In v_rep.ResultCapital
                        v_kontos.Append(v_item.Item2 & ";")
                    Next
                    .Blocks.Add(New Paragraph(New Run(v_kontos.ToString)) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    v_kontos = Nothing

                    'Print the diagram if activated:
                    If v_rep.print_diagram Then
                        .Blocks.Add(New Paragraph(New Run("Grafische Übersicht")))
                        Dim v_diagram As New CartesianChart With {.LegendLocation = LegendLocation.Top, .Series = v_rep.Chart_Series, .Height = 200, .Width = 700, .HorizontalAlignment = HorizontalAlignment.Center, .DisableAnimations = True}
                        v_diagram.AxisX.Add(New Axis With {.Title = "Beträge", .Labels = v_rep.Chart_Labels})
                        v_diagram.AxisY.Add(New Axis With {.Title = "Wert (€)", .LabelFormatter = v_rep.Chart_Formatter})
                        Dim viewbox = New Viewbox()
                        viewbox.Child = v_diagram
                        viewbox.Measure(v_diagram.RenderSize)
                        viewbox.Arrange(New Rect(New Point(0, 0), v_diagram.RenderSize))
                        v_diagram.Update(True, True)
                        viewbox.UpdateLayout()
                        .Blocks.Add(New BlockUIContainer(viewbox))
                    End If

                    'Now Add the Results Grid:
                    .Blocks.Add(New Paragraph(New Run("Übersicht")))
                    'Grid Header:
                    Dim v_grid_header As New Grid With {.ShowGridLines = False}
                    With v_grid_header
                        .RowDefinitions.Add(New RowDefinition)
                        For Each v_str As KeyValuePair(Of String, Integer) In v_header
                            .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                            Dim v_border As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = v_headerbackgroundbrush}
                            Dim v_lbl As New Label With {.Content = v_str.Key, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_header_fontsize}
                            v_border.Child = v_lbl
                            Grid.SetColumn(v_border, .ColumnDefinitions.Count - 1)
                            Grid.SetRow(v_border, 0)
                            .Children.Add(v_border)
                            v_lbl = Nothing
                        Next
                    End With
                    .Blocks.Add(New BlockUIContainer(v_grid_header))
                    For Each v_item As Tuple(Of String, String, Decimal) In v_rep.ResultCapital
                        Dim v_grid As New Grid With {.ShowGridLines = False}
                        With v_grid
                            .RowDefinitions.Add(New RowDefinition)
                            For Each v_str As KeyValuePair(Of String, Integer) In v_header
                                .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                            Next

                            'Add the folder name:
                            Dim v_border As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl As New Label With {.Content = v_item.Item1, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border.Child = v_lbl
                            Grid.SetColumn(v_border, 0)
                            Grid.SetRow(v_border, 0)
                            .Children.Add(v_border)
                            v_lbl = Nothing
                            v_border = Nothing

                            'Add the Konto name:
                            Dim v_border_kto As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl_kto As New Label With {.Content = v_item.Item2, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border_kto.Child = v_lbl_kto
                            Grid.SetColumn(v_border_kto, 1)
                            Grid.SetRow(v_border_kto, 0)
                            .Children.Add(v_border_kto)
                            v_lbl_kto = Nothing
                            v_border_kto = Nothing

                            'Add the Value:
                            Dim v_border_val As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl_val As New Label With {.Content = v_item.Item3.ToString("C"), .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border_val.Child = v_lbl_val
                            Grid.SetColumn(v_border_val, 2)
                            Grid.SetRow(v_border_val, 0)
                            .Children.Add(v_border_val)
                            v_lbl_val = Nothing
                            v_border_val = Nothing
                        End With
                        .Blocks.Add(New BlockUIContainer(v_grid))
                        v_grid = Nothing
                        v_rowcount += 1
                    Next
                End With
            End With
            Return True
        End Function

        Private Function RepFoldertransEinAusPrintPrepare(v_rep As report_base_v01, v_bezeichnung As String) As Boolean
            If v_rep Is Nothing Then Return False
            With v_rep
                If .ResultFolderTrans Is Nothing OrElse .ResultFolderTrans.Count = 0 Then Return False

                Dim v_rowcount As Integer = 1
                Dim v_oddrowbrush As Brush = SettingsCurrent.color_background_oddrow_brush
                Dim v_justrowbrush As Brush = SettingsCurrent.color_background_justrow_brush
                Dim v_headerbackgroundbrush As Brush = SettingsCurrent.color_background_header_brush
                Dim v_header_fontsize As Double = SettingsCurrent.header_fontsize - 10
                Dim v_row_fontsize As Double = SettingsCurrent.row_fontsize - 4

                'Create the report header:
                .PrintDocument.Blocks.Clear()

                'Headers for the Konto Table:
                Dim v_header As New Dictionary(Of String, Integer) From {
                    New KeyValuePair(Of String, Integer)(v_bezeichnung, 20),
                    New KeyValuePair(Of String, Integer)("Betrag", 20)}

                'Define the report:
                With .PrintDocument
                    .Blocks.Clear()
                    'Header:
                    .Blocks.Add(New Paragraph(New Run(String.Format("{0} Bericht - {1}", v_bezeichnung, v_rep.rep_bezeichnung))) With {.FontSize = SettingsCurrent.header_fontsize})
                    .Blocks.Add(New Paragraph(New Run(Date.Now.ToShortDateString)) With {.FontSize = 12, .TextAlignment = TextAlignment.Right})
                    .Blocks.Add(New Paragraph(New Run("Kriterien")))
                    .Blocks.Add(New Paragraph(New Run(String.Format("Datumsbereich von {0} bis {1}", v_rep.date_start.ToShortDateString, v_rep.date_end.ToShortDateString))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    If Not v_bezeichnung = "Ein/Aus" Then
                        Dim v_folders As New StringBuilder
                        v_folders.Append("Berücksichtigte Ordner:")
                        For Each v_item As Tuple(Of String, Decimal) In v_rep.ResultFolderTrans
                            v_folders.Append(v_item.Item1 & ";")
                        Next
                        .Blocks.Add(New Paragraph(New Run(v_folders.ToString)) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                        v_folders = Nothing
                    End If
                    'Print the diagram if activated:
                    If v_rep.print_diagram Then
                        .Blocks.Add(New Paragraph(New Run("Grafische Übersicht")))
                        Dim v_diagram As New CartesianChart With {.LegendLocation = LegendLocation.Top, .Series = v_rep.Chart_Series, .Height = 200, .Width = 700, .HorizontalAlignment = HorizontalAlignment.Center, .DisableAnimations = True}
                        v_diagram.AxisX.Add(New Axis With {.Title = "Beträge", .Labels = v_rep.Chart_Labels})
                        v_diagram.AxisY.Add(New Axis With {.Title = "Wert (€)", .LabelFormatter = v_rep.Chart_Formatter})
                        Dim viewbox = New Viewbox()
                        viewbox.Child = v_diagram
                        viewbox.Measure(v_diagram.RenderSize)
                        viewbox.Arrange(New Rect(New Point(0, 0), v_diagram.RenderSize))
                        v_diagram.Update(True, True)
                        viewbox.UpdateLayout()
                        .Blocks.Add(New BlockUIContainer(viewbox))
                    End If

                    'Now Add the Results Grid:
                    .Blocks.Add(New Paragraph(New Run("Übersicht")))
                    'Grid Header:
                    Dim v_grid_header As New Grid With {.ShowGridLines = False}
                    With v_grid_header
                        .RowDefinitions.Add(New RowDefinition)
                        For Each v_str As KeyValuePair(Of String, Integer) In v_header
                            .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                            Dim v_border As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = v_headerbackgroundbrush}
                            Dim v_lbl As New Label With {.Content = v_str.Key, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_header_fontsize}
                            v_border.Child = v_lbl
                            Grid.SetColumn(v_border, .ColumnDefinitions.Count - 1)
                            Grid.SetRow(v_border, 0)
                            .Children.Add(v_border)
                            v_lbl = Nothing
                        Next
                    End With
                    .Blocks.Add(New BlockUIContainer(v_grid_header))
                    For Each v_item As Tuple(Of String, Decimal) In v_rep.ResultFolderTrans
                        Dim v_grid As New Grid With {.ShowGridLines = False}
                        With v_grid
                            .RowDefinitions.Add(New RowDefinition)
                            For Each v_str As KeyValuePair(Of String, Integer) In v_header
                                .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                            Next

                            'Add the folder name:
                            Dim v_border As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl As New Label With {.Content = v_item.Item1, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border.Child = v_lbl
                            Grid.SetColumn(v_border, 0)
                            Grid.SetRow(v_border, 0)
                            .Children.Add(v_border)
                            v_lbl = Nothing
                            v_border = Nothing

                            'Add the Value:
                            Dim v_border_val As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl_val As New Label With {.Content = v_item.Item2.ToString("C"), .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border_val.Child = v_lbl_val
                            Grid.SetColumn(v_border_val, 1)
                            Grid.SetRow(v_border_val, 0)
                            .Children.Add(v_border_val)
                            v_lbl_val = Nothing
                            v_border_val = Nothing
                        End With
                        .Blocks.Add(New BlockUIContainer(v_grid))
                        v_grid = Nothing
                        v_rowcount += 1
                    Next
                End With
            End With
            Return True
        End Function

        Private Function RepCatsTopCatsPrintPrepare(v_rep As report_base_v01, v_bezeichnung As String, v_istop As Boolean) As Boolean
            If v_rep Is Nothing Then Return False
            With v_rep
                If .ResultCapital Is Nothing OrElse .ResultCapital.Count = 0 Then Return False

                Dim v_rowcount As Integer = 1
                Dim v_oddrowbrush As Brush = SettingsCurrent.color_background_oddrow_brush
                Dim v_justrowbrush As Brush = SettingsCurrent.color_background_justrow_brush
                Dim v_headerbackgroundbrush As Brush = SettingsCurrent.color_background_header_brush
                Dim v_header_fontsize As Double = SettingsCurrent.header_fontsize - 10
                Dim v_row_fontsize As Double = SettingsCurrent.row_fontsize - 4

                'Create the report header:
                .PrintDocument.Blocks.Clear()

                'Headers for the Konto Table:
                Dim v_header As New Dictionary(Of String, Integer) From {
                    New KeyValuePair(Of String, Integer)("Übergeordnet", 20),
                    New KeyValuePair(Of String, Integer)(v_bezeichnung, 20),
                    New KeyValuePair(Of String, Integer)("Betrag", 20)}

                'Define the report:
                With .PrintDocument
                    .Blocks.Clear()
                    'Header:
                    If v_istop Then
                        .Blocks.Add(New Paragraph(New Run(String.Format("Top {0} Bericht - {1}", v_bezeichnung, v_rep.rep_bezeichnung))) With {.FontSize = SettingsCurrent.header_fontsize})
                    Else
                        .Blocks.Add(New Paragraph(New Run(String.Format("{0} Bericht - {1}", v_bezeichnung, v_rep.rep_bezeichnung))) With {.FontSize = SettingsCurrent.header_fontsize})
                    End If
                    .Blocks.Add(New Paragraph(New Run(Date.Now.ToShortDateString)) With {.FontSize = 12, .TextAlignment = TextAlignment.Right})
                    .Blocks.Add(New Paragraph(New Run("Kriterien")))
                    .Blocks.Add(New Paragraph(New Run(String.Format("Datumsbereich von {0} bis {1}", v_rep.date_start.ToShortDateString, v_rep.date_end.ToShortDateString))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    If v_istop Then
                        .Blocks.Add(New Paragraph(New Run(String.Format("max Anzahl: {0}", v_rep.count.ToString))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    Else
                        Dim v_cats As New StringBuilder
                        v_cats.Append(String.Format("Berücksichtigte {0}:", v_bezeichnung))
                        For Each v_item As Tuple(Of String, String, Decimal) In v_rep.ResultCapital
                            v_cats.Append(v_item.Item1 & " - " & v_item.Item2 & "; ")
                        Next
                        .Blocks.Add(New Paragraph(New Run(v_cats.ToString)) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                        v_cats = Nothing
                    End If

                    'Print the diagram if activated:
                    If v_rep.print_diagram Then
                        .Blocks.Add(New Paragraph(New Run("Grafische Übersicht")))
                        'Cartesian chart:
                        Dim v_diagram As New CartesianChart With {.LegendLocation = LegendLocation.Top, .Series = v_rep.Chart_Series, .Height = 200, .Width = 700, .HorizontalAlignment = HorizontalAlignment.Center, .DisableAnimations = True}
                        v_diagram.AxisX.Add(New Axis With {.Title = "Beträge", .Labels = v_rep.Chart_Labels})
                        v_diagram.AxisY.Add(New Axis With {.Title = "Wert (€)", .LabelFormatter = v_rep.Chart_Formatter})
                        Dim viewbox As New Viewbox With {.Child = v_diagram}
                        viewbox.Measure(v_diagram.RenderSize)
                        viewbox.Arrange(New Rect(New Point(0, 0), v_diagram.RenderSize))
                        v_diagram.Update(True, True)
                        viewbox.UpdateLayout()
                        .Blocks.Add(New BlockUIContainer(viewbox))
                        'Piechart:
                        Dim v_pie_diagram As New PieChart With {.LegendLocation = LegendLocation.Top, .Hoverable = False, .Series = v_rep.PieChart_Series, .Height = 200, .Width = 700, .HorizontalAlignment = HorizontalAlignment.Center, .DisableAnimations = True}
                        Dim viewbox_pie As New Viewbox With {.Child = v_pie_diagram}
                        viewbox_pie.Measure(v_pie_diagram.RenderSize)
                        viewbox_pie.Arrange(New Rect(New Point(0, 0), v_pie_diagram.RenderSize))
                        v_pie_diagram.Update(True, True)
                        viewbox_pie.UpdateLayout()
                        .Blocks.Add(New Paragraph(New Run()))
                        .Blocks.Add(New BlockUIContainer(viewbox_pie))
                    End If

                    'Now Add the Results Grid:
                    .Blocks.Add(New Paragraph(New Run("Übersicht")))
                    'Grid Header:
                    Dim v_grid_header As New Grid With {.ShowGridLines = False}
                    With v_grid_header
                        .RowDefinitions.Add(New RowDefinition)
                        For Each v_str As KeyValuePair(Of String, Integer) In v_header
                            .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                            Dim v_border As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = v_headerbackgroundbrush}
                            Dim v_lbl As New Label With {.Content = v_str.Key, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_header_fontsize}
                            v_border.Child = v_lbl
                            Grid.SetColumn(v_border, .ColumnDefinitions.Count - 1)
                            Grid.SetRow(v_border, 0)
                            .Children.Add(v_border)
                            v_lbl = Nothing
                        Next
                    End With
                    .Blocks.Add(New BlockUIContainer(v_grid_header))
                    For Each v_item As Tuple(Of String, String, Decimal) In v_rep.ResultCapital
                        Dim v_grid As New Grid With {.ShowGridLines = False}
                        With v_grid
                            .RowDefinitions.Add(New RowDefinition)
                            For Each v_str As KeyValuePair(Of String, Integer) In v_header
                                .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                            Next

                            'Add the parent category name:
                            Dim v_border As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl As New Label With {.Content = v_item.Item1, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border.Child = v_lbl
                            Grid.SetColumn(v_border, 0)
                            Grid.SetRow(v_border, 0)
                            .Children.Add(v_border)
                            v_lbl = Nothing
                            v_border = Nothing

                            'Add the category name:
                            Dim v_border_cat As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl_cat As New Label With {.Content = v_item.Item2, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border_cat.Child = v_lbl_cat
                            Grid.SetColumn(v_border_cat, 1)
                            Grid.SetRow(v_border_cat, 0)
                            .Children.Add(v_border_cat)
                            v_lbl_cat = Nothing
                            v_border_cat = Nothing

                            'Add the Value:
                            Dim v_border_val As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl_val As New Label With {.Content = v_item.Item3.ToString("C"), .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border_val.Child = v_lbl_val
                            Grid.SetColumn(v_border_val, 2)
                            Grid.SetRow(v_border_val, 0)
                            .Children.Add(v_border_val)
                            v_lbl_val = Nothing
                            v_border_val = Nothing
                        End With
                        .Blocks.Add(New BlockUIContainer(v_grid))
                        v_grid = Nothing
                        v_rowcount += 1
                    Next
                End With
            End With
            Return True
        End Function

        Private Function RepReceiverPersonTopReceiverTopPersonPrintPrepare(v_rep As report_base_v01, v_bezeichnung As String, v_istop As Boolean) As Boolean
            If v_rep Is Nothing Then Return False
            With v_rep
                If .ResultFolderTrans Is Nothing OrElse .ResultFolderTrans.Count = 0 Then Return False

                Dim v_rowcount As Integer = 1
                Dim v_oddrowbrush As Brush = SettingsCurrent.color_background_oddrow_brush
                Dim v_justrowbrush As Brush = SettingsCurrent.color_background_justrow_brush
                Dim v_headerbackgroundbrush As Brush = SettingsCurrent.color_background_header_brush
                Dim v_header_fontsize As Double = SettingsCurrent.header_fontsize - 10
                Dim v_row_fontsize As Double = SettingsCurrent.row_fontsize - 4

                'Create the report header:
                .PrintDocument.Blocks.Clear()

                'Headers for the Konto Table:
                Dim v_header As New Dictionary(Of String, Integer) From {
                    New KeyValuePair(Of String, Integer)(v_bezeichnung, 20),
                    New KeyValuePair(Of String, Integer)("Betrag", 20)}

                'Define the report:
                With .PrintDocument
                    .Blocks.Clear()
                    'Header:
                    If v_istop Then
                        .Blocks.Add(New Paragraph(New Run(String.Format("Top {0} Bericht - {1}", v_bezeichnung, v_rep.rep_bezeichnung))) With {.FontSize = SettingsCurrent.header_fontsize})
                    Else
                        .Blocks.Add(New Paragraph(New Run(String.Format("{0} Bericht - {1}", v_bezeichnung, v_rep.rep_bezeichnung))) With {.FontSize = SettingsCurrent.header_fontsize})
                    End If
                    .Blocks.Add(New Paragraph(New Run(Date.Now.ToShortDateString)) With {.FontSize = 12, .TextAlignment = TextAlignment.Right})
                    .Blocks.Add(New Paragraph(New Run("Kriterien")))
                    .Blocks.Add(New Paragraph(New Run(String.Format("Datumsbereich von {0} bis {1}", v_rep.date_start.ToShortDateString, v_rep.date_end.ToShortDateString))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    If v_istop Then
                        .Blocks.Add(New Paragraph(New Run(String.Format("max Anzahl: {0}", v_rep.count.ToString))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    Else
                        Dim v_criteria As New StringBuilder
                        v_criteria.Append(String.Format("Berücksichtigte {0}:", v_bezeichnung))
                        For Each v_item As Tuple(Of String, Decimal) In v_rep.ResultFolderTrans
                            v_criteria.Append(v_item.Item1 & "; ")
                        Next
                        .Blocks.Add(New Paragraph(New Run(v_criteria.ToString)) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                        v_criteria = Nothing
                    End If
                    'Print the diagram if activated:
                    If v_rep.print_diagram Then
                        .Blocks.Add(New Paragraph(New Run("Grafische Übersicht")))
                        'Cartesian chart:
                        Dim v_diagram As New CartesianChart With {.LegendLocation = LegendLocation.Top, .Series = v_rep.Chart_Series, .Height = 200, .Width = 700, .HorizontalAlignment = HorizontalAlignment.Center, .DisableAnimations = True}
                        v_diagram.AxisX.Add(New Axis With {.Title = "Beträge", .Labels = v_rep.Chart_Labels})
                        v_diagram.AxisY.Add(New Axis With {.Title = "Wert (€)", .LabelFormatter = v_rep.Chart_Formatter})
                        Dim viewbox As New Viewbox With {.Child = v_diagram}
                        viewbox.Measure(v_diagram.RenderSize)
                        viewbox.Arrange(New Rect(New Point(0, 0), v_diagram.RenderSize))
                        v_diagram.Update(True, True)
                        viewbox.UpdateLayout()
                        .Blocks.Add(New BlockUIContainer(viewbox))
                        'Piechart:
                        Dim v_pie_diagram As New PieChart With {.LegendLocation = LegendLocation.Top, .Hoverable = False, .Series = v_rep.PieChart_Series, .Height = 200, .Width = 700, .HorizontalAlignment = HorizontalAlignment.Center, .DisableAnimations = True}
                        Dim viewbox_pie As New Viewbox With {.Child = v_pie_diagram}
                        viewbox_pie.Measure(v_pie_diagram.RenderSize)
                        viewbox_pie.Arrange(New Rect(New Point(0, 0), v_pie_diagram.RenderSize))
                        v_pie_diagram.Update(True, True)
                        viewbox_pie.UpdateLayout()
                        .Blocks.Add(New Paragraph(New Run()))
                        .Blocks.Add(New BlockUIContainer(viewbox_pie))
                    End If

                    'Now Add the Results Grid:
                    .Blocks.Add(New Paragraph(New Run("Übersicht")))
                    'Grid Header:
                    Dim v_grid_header As New Grid With {.ShowGridLines = False}
                    With v_grid_header
                        .RowDefinitions.Add(New RowDefinition)
                        For Each v_str As KeyValuePair(Of String, Integer) In v_header
                            .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                            Dim v_border As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = v_headerbackgroundbrush}
                            Dim v_lbl As New Label With {.Content = v_str.Key, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_header_fontsize}
                            v_border.Child = v_lbl
                            Grid.SetColumn(v_border, .ColumnDefinitions.Count - 1)
                            Grid.SetRow(v_border, 0)
                            .Children.Add(v_border)
                            v_lbl = Nothing
                        Next
                    End With
                    .Blocks.Add(New BlockUIContainer(v_grid_header))
                    For Each v_item As Tuple(Of String, Decimal) In v_rep.ResultFolderTrans
                        Dim v_grid As New Grid With {.ShowGridLines = False}
                        With v_grid
                            .RowDefinitions.Add(New RowDefinition)
                            For Each v_str As KeyValuePair(Of String, Integer) In v_header
                                .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                            Next

                            'Add the category name:
                            Dim v_border_cat As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl_cat As New Label With {.Content = v_item.Item1, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border_cat.Child = v_lbl_cat
                            Grid.SetColumn(v_border_cat, 0)
                            Grid.SetRow(v_border_cat, 0)
                            .Children.Add(v_border_cat)
                            v_lbl_cat = Nothing
                            v_border_cat = Nothing

                            'Add the Value:
                            Dim v_border_val As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                            Dim v_lbl_val As New Label With {.Content = v_item.Item2.ToString("C"), .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                            v_border_val.Child = v_lbl_val
                            Grid.SetColumn(v_border_val, 1)
                            Grid.SetRow(v_border_val, 0)
                            .Children.Add(v_border_val)
                            v_lbl_val = Nothing
                            v_border_val = Nothing
                        End With
                        .Blocks.Add(New BlockUIContainer(v_grid))
                        v_grid = Nothing
                        v_rowcount += 1
                    Next
                End With
            End With
            Return True
        End Function

        Private Function RepPeriodsPrintPrepare(v_rep As report_base_v01, v_bezeichnung As String, v_istop As Boolean) As Boolean
            If v_rep Is Nothing Then Return False
            With v_rep
                If .ResultPeriodsHeader Is Nothing OrElse .ResultPeriodsPosCats Is Nothing OrElse .ResultPeriodsNegCats Is Nothing OrElse .ResultPeriodsHeader.Count = 0 Then Return False

                Dim v_rowcount As Integer = 1
                Dim v_oddrowbrush As Brush = SettingsCurrent.color_background_oddrow_brush
                Dim v_justrowbrush As Brush = SettingsCurrent.color_background_justrow_brush
                Dim v_headerbackgroundbrush As Brush = SettingsCurrent.color_background_header_brush
                Dim v_header_fontsize As Double = SettingsCurrent.header_fontsize - 10
                Dim v_row_fontsize As Double = SettingsCurrent.row_fontsize - 4

                'Define the report:
                With .PrintDocument

                    .Blocks.Clear()
                    'Header:
                    .Blocks.Add(New Paragraph(New Run(String.Format("{0} Bericht - {1}", v_bezeichnung, v_rep.rep_bezeichnung))) With {.FontSize = SettingsCurrent.header_fontsize})
                    .Blocks.Add(New Paragraph(New Run(Date.Now.ToShortDateString)) With {.FontSize = 12, .TextAlignment = TextAlignment.Right})
                    .Blocks.Add(New Paragraph(New Run("Kriterien")))
                    .Blocks.Add(New Paragraph(New Run(String.Format("Datumsbereich ab {0}", v_rep.date_start.ToShortDateString))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    'Get the name of the period:
                    Dim v_selected_timeframe As cls_timeframe = __timeframes_periods_report.FirstOrDefault(Function(x) x.Id = v_rep.selected_period_period)
                    If v_selected_timeframe IsNot Nothing Then .Blocks.Add(New Paragraph(New Run(String.Format("Periodenspanne {0}", v_selected_timeframe.bezeichnung))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    v_selected_timeframe = Nothing
                    'Amount of periods:
                    .Blocks.Add(New Paragraph(New Run(String.Format("Perioden-Anzahl: {0}", v_rep.count.ToString))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    'Summarized or not:
                    .Blocks.Add(New Paragraph(New Run(String.Format("Kategorien Summenbildung: {0}", If(v_rep.positive, "ja", "nein")))) With {.FontSize = 12, .TextAlignment = TextAlignment.Left})
                    'Print the diagram if activated:
                    If v_rep.print_diagram Then
                        .Blocks.Add(New Paragraph(New Run("Grafische Übersicht")))
                        'Cartesian chart:
                        Dim v_diagram As New CartesianChart With {.LegendLocation = LegendLocation.Top, .Series = v_rep.Chart_Series, .Height = 200, .Width = 1000, .HorizontalAlignment = HorizontalAlignment.Center, .DisableAnimations = True}
                        v_diagram.AxisX.Add(New Axis With {.Title = "Perioden", .Labels = v_rep.Chart_Labels})
                        v_diagram.AxisY.Add(New Axis With {.Title = "Wert (€)", .LabelFormatter = v_rep.Chart_Formatter})
                        Dim viewbox As New Viewbox With {.Child = v_diagram}
                        viewbox.Measure(v_diagram.RenderSize)
                        viewbox.Arrange(New Rect(New Point(0, 0), v_diagram.RenderSize))
                        v_diagram.Update(True, True)
                        viewbox.UpdateLayout()
                        .Blocks.Add(New BlockUIContainer(viewbox))
                    End If

                    'Now Add the Results Grid:
                    .Blocks.Add(New Paragraph(New Run("Einnahmen")))
                    'Grid Header:
                    '        Dim v_grid_header As New Grid With {.ShowGridLines = False}
                    '        With v_grid_header
                    '            .RowDefinitions.Add(New RowDefinition)
                    '            For Each v_str As KeyValuePair(Of String, Integer) In v_header
                    '                .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                    '                Dim v_border As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = v_headerbackgroundbrush}
                    '                Dim v_lbl As New Label With {.Content = v_str.Key, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_header_fontsize}
                    '                v_border.Child = v_lbl
                    '                Grid.SetColumn(v_border, .ColumnDefinitions.Count - 1)
                    '                Grid.SetRow(v_border, 0)
                    '                .Children.Add(v_border)
                    '                v_lbl = Nothing
                    '            Next
                    '        End With
                    '        .Blocks.Add(New BlockUIContainer(v_grid_header))
                    '        For Each v_item As Tuple(Of String, Decimal) In v_rep.ResultFolderTrans
                    '            Dim v_grid As New Grid With {.ShowGridLines = False}
                    '            With v_grid
                    '                .RowDefinitions.Add(New RowDefinition)
                    '                For Each v_str As KeyValuePair(Of String, Integer) In v_header
                    '                    .ColumnDefinitions.Add(New ColumnDefinition With {.Width = New GridLength(v_str.Value, GridUnitType.Star)})
                    '                Next

                    '                'Add the category name:
                    '                Dim v_border_cat As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                    '                Dim v_lbl_cat As New Label With {.Content = v_item.Item1, .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                    '                v_border_cat.Child = v_lbl_cat
                    '                Grid.SetColumn(v_border_cat, 0)
                    '                Grid.SetRow(v_border_cat, 0)
                    '                .Children.Add(v_border_cat)
                    '                v_lbl_cat = Nothing
                    '                v_border_cat = Nothing

                    '                'Add the Value:
                    '                Dim v_border_val As New Border With {.BorderBrush = Brushes.Black, .BorderThickness = New Thickness(1, 1, 1, 1), .Padding = New Thickness(3), .Background = If(v_rowcount Mod 2 = 0, v_justrowbrush, v_oddrowbrush)}
                    '                Dim v_lbl_val As New Label With {.Content = v_item.Item2.ToString("C"), .VerticalAlignment = VerticalAlignment.Center, .HorizontalAlignment = HorizontalAlignment.Center, .FontSize = v_row_fontsize}
                    '                v_border_val.Child = v_lbl_val
                    '                Grid.SetColumn(v_border_val, 1)
                    '                Grid.SetRow(v_border_val, 0)
                    '                .Children.Add(v_border_val)
                    '                v_lbl_val = Nothing
                    '                v_border_val = Nothing
                    '            End With
                    '            .Blocks.Add(New BlockUIContainer(v_grid))
                    '            v_grid = Nothing
                    '            v_rowcount += 1
                    '        Next
                End With
            End With
                Return True
        End Function

        Private Sub ReportDoPrint(v_rep As report_base_v01, v_portrait As Boolean)
            If v_rep Is Nothing Then Exit Sub
            If v_rep.PrintDocument Is Nothing Then Exit Sub

            Dim v_print_dia As New PrintDialog
            Dim v_print_server As New LocalPrintServer
            Dim v_print_queue As PrintQueue = v_print_server.DefaultPrintQueue
            Dim v_print_ticket As PrintTicket = v_print_queue.UserPrintTicket
            Dim v_pm_size As PageMediaSize = New PageMediaSize(v_rep.PrintDocument.PageWidth, v_rep.PrintDocument.PageHeight)
            If v_portrait Then
                v_print_ticket.PageOrientation = PageOrientation.Portrait
            Else
                v_pm_size = New PageMediaSize(v_rep.PrintDocument.PageHeight, v_rep.PrintDocument.PageWidth)
                v_print_ticket.PageOrientation = PageOrientation.Landscape
            End If
            v_print_ticket.PageMediaSize = v_pm_size


            Dim v_docpag As IDocumentPaginatorSource = TryCast(v_rep.PrintDocument, IDocumentPaginatorSource)

            If v_docpag Is Nothing Then
                cls_logger.v_logger.Error("Umwandlung von v_rep.PrintDocument in Paginator failed")
                Dim vMainWindow As MainWindow = CType(Application.Current.MainWindow, MainWindow)
                vMainWindow.ShowMessageAsync("Fehler", "Fehler bei Ausdruck, Report konnte nicht gedruckt werden", MessageDialogStyle.Affirmative)
                Exit Sub
            End If

            Dim v_imageablearea As PrintDocumentImageableArea = Nothing
            Dim v_docwriter As Xps.XpsDocumentWriter = PrintQueue.CreateXpsDocumentWriter(v_imageablearea)

            v_docpag.DocumentPaginator.ComputePageCount()
            Dim paginator As DocumentPaginator = v_docpag.DocumentPaginator
            If v_portrait Then
                paginator = New DocumentPaginatorWrapper(v_docpag.DocumentPaginator, New Size(768, 1104), New Size(20, 45))
            Else
                paginator = New DocumentPaginatorWrapper(v_docpag.DocumentPaginator, New Size(1104, 768), New Size(20, 45))
            End If
            If v_docwriter IsNot Nothing AndAlso v_imageablearea IsNot Nothing Then
                v_docwriter.Write(paginator, v_print_ticket)
            End If
        End Sub


#End Region

        Private Async Sub ReportSave(v_rep As report_base_v01)
            If v_rep Is Nothing OrElse v_rep.hasErrors Then
                Dim vMainWindow As MainWindow = CType(Application.Current.MainWindow, MainWindow)
                Dim v_result As MessageDialogResult = Await vMainWindow.ShowMessageAsync("Änderungen können nicht gespeichert werden", "Die Einstellungen des Berichts sind fehlerhaft, speichern nicht möglich", MessageDialogStyle.AffirmativeAndNegative, New MetroDialogSettings With {.NegativeButtonText = "Änderungen verwerfen"})
                If Not v_result = MessageDialogResult.Affirmative Then v_rep.CancelEdit()
                Exit Sub
            Else
                v_rep.EndEdit()
                dbo.SaveChanges()
            End If
        End Sub

        Private Function ReportSaveCanExecute(v_rep As report_base_v01) As Boolean
            If v_rep Is Nothing Then Return False
            Return True
        End Function


#End Region
    End Class
End Namespace